/**
 * Copyright 2017 Anders Lunde, Robin Sjvoll
 *
 * This code is developed under an experimental paradigm, thus it is important to reuse it with caution. 
 */
package no.ntnu.es.bb.schema;

import java.util.List;

import org.apache.log4j.Logger;

import no.ntnu.es.bb.schema.Evaluation;
import blackboard.persist.Id;
import blackboard.persist.PersistenceException;
import blackboard.persist.KeyNotFoundException;
import blackboard.persist.dao.impl.SimpleDAO;
import blackboard.persist.impl.SimpleSelectQuery;
import blackboard.util.StringUtil;

public class EvaluationDAO extends SimpleDAO<Evaluation> {
	private static final Logger LOGGER = Logger.getLogger( EvaluationDAO.class.getName() );
	public EvaluationDAO() {
		super(Evaluation.class);
	}

	public EvaluationDAO(Class<Evaluation> cls) {
		super(cls);
	}

	public List<Evaluation> loadAll() {
		return getDAOSupport().loadAll();
	}

	public List<Evaluation> searchById(String id) 
			throws KeyNotFoundException {
		if (!StringUtil.isEmpty(id)) {
			SimpleSelectQuery query = new SimpleSelectQuery(getDAOSupport().getMap());
			//Change column name to whatever the name is in the bean
			query.addWhere("pk1", id);
			return getDAOSupport().loadList(query);
		}
		return null;  
	} 
	
	public Evaluation searchByUserIDAndSubmissionID(Integer userID, Integer submissionID) 
			throws KeyNotFoundException {
		LOGGER.debug("User ID and Submission ID "+ userID +" # " + submissionID);
		if (userID != 0 && submissionID != 0) {
			SimpleSelectQuery query = new SimpleSelectQuery(getDAOSupport().getMap());
			//Change column name to whatever the name is in the bean
			query.addWhere("reviewerUserId", userID);
			query.addWhere("peerSubmissiontId", submissionID);
//			return getDAOSupport().loadList(query).get(0);
			return getDAOSupport().load(query);
		}
		return null;  
	} 
	
	public Evaluation searchByGroupIDAndSubmissionID(Integer groupID, Integer submissionID) 
			throws KeyNotFoundException {
		LOGGER.debug("Group ID and Submission ID "+ groupID +" # " + submissionID);
		if (groupID != 0 && submissionID != 0) {
			SimpleSelectQuery query = new SimpleSelectQuery(getDAOSupport().getMap());
			//Change column name to whatever the name is in the bean
			query.addWhere("groupEvaluationId", groupID);
			query.addWhere("peerSubmissiontId", submissionID);
//			return getDAOSupport().loadList(query).get(0);
			return getDAOSupport().load(query);
		}
		return null;  
	} 
	
	public List<Evaluation> findAllWithSubmissiontID(Integer submissionID) 
			throws KeyNotFoundException {
		LOGGER.debug("Find all Evaluations with SubmissionID: " + submissionID);
		if (submissionID != 0) {
			SimpleSelectQuery query = new SimpleSelectQuery(getDAOSupport().getMap());
			query.addWhere("peerSubmissiontId", submissionID);
			return getDAOSupport().loadList(query);
		}
		return null;  
	} 
	
	//Currently loads the last element in the list (last added)
    public Evaluation loadById(long id) {
        Evaluation evaluation = null; 
        try {
        	 Id realId = Id.generateId(Evaluation.DATA_TYPE, id);
        	 evaluation = getDAOSupport().loadById(realId);
		} catch (PersistenceException e1) {
			LOGGER.debug("Failed to load courseID! ID: "+ id +" # " + e1);
			e1.printStackTrace();
		}
       
        LOGGER.debug("Evaluation Loaded succsessfully "+ id +" # " + evaluation);
        if(evaluation!=null){
        	return evaluation;
        }
        else return null;
    } 
    
  //Currently loads the last element in the list (last added)
    public Evaluation loadLast() {
        List<Evaluation> globalSettings;
        globalSettings = getDAOSupport().loadAll();
        if(globalSettings!=null&&!globalSettings.isEmpty()){
            //DEBUGGING
        	for (Evaluation evaluation : globalSettings) {
        		LOGGER.debug("A#"+ evaluation);
			}
        return globalSettings.get(globalSettings.size()-1);
        }
        else return null;
    } 
    
    public void save(Evaluation globalSettings) {
        System.out.println(globalSettings);        
                
        getDAOSupport().persist(globalSettings);
    }
    
    public boolean isInstalled(){
        List<Evaluation> globalSettings = null;
        globalSettings = getDAOSupport().loadAll();
        
   
        if (globalSettings.size() > 0) {
            return true;
        }
        return false;
    }
    
    public void writeDefaults() {
    	Evaluation globalSettings = new Evaluation();
        
        getDAOSupport().persist(globalSettings);
        
    }
}
