/**
 * Copyright 2017 Anders Lunde, Robin Sjvoll
 *
 * This code is developed under an experimental paradigm, thus it is important to reuse it with caution. 
 */
package no.ntnu.es.bb.schema;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.StringTokenizer;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.ModelAndView;

import no.ntnu.es.bb.schema.Assessment;
import no.ntnu.es.bb.schema.AssessmentDAO;
import no.ntnu.es.bb.schema.rubric.Rubric;
import no.ntnu.es.bb.schema.rubric.RubricCell;
import no.ntnu.es.bb.schema.rubric.RubricCellDAO;
import no.ntnu.es.bb.schema.rubric.RubricColumn;
import no.ntnu.es.bb.schema.rubric.RubricColumnDAO;
import no.ntnu.es.bb.schema.rubric.RubricDAO;
import no.ntnu.es.bb.schema.rubric.RubricRow;
import no.ntnu.es.bb.schema.rubric.RubricRowDAO;
import no.ntnu.es.bb.schema.rubric.RubricRowEval;
import no.ntnu.es.bb.schema.rubric.RubricRowEvalDAO;
import blackboard.base.FormattedText;
import blackboard.data.ReceiptOptions;
import blackboard.data.content.Content;
import blackboard.data.course.Course;
import blackboard.data.course.CourseMembership;
import blackboard.data.course.Group;
import blackboard.data.course.GroupMembership;
import blackboard.data.user.User;
import blackboard.persist.DataType;
import blackboard.persist.Id;
import blackboard.persist.KeyNotFoundException;
import blackboard.persist.PersistenceException;
import blackboard.persist.PersistenceRuntimeException;
import blackboard.persist.course.CourseMembershipDbLoader;
import blackboard.persist.course.GroupDbLoader;
import blackboard.persist.user.UserDbLoader;
import blackboard.platform.context.Context;
import blackboard.platform.context.ContextManager;
import blackboard.platform.context.ContextManagerFactory;
import blackboard.platform.filesystem.FileSystemException;
import blackboard.platform.filesystem.manager.CourseFileManager;
import blackboard.platform.gradebook2.GradableItem;
import blackboard.platform.gradebook2.GradableItemManager;
import blackboard.platform.gradebook2.GradeDetail;
import blackboard.platform.gradebook2.GradebookManagerFactory;
import blackboard.platform.gradebook2.impl.GradeDetailDAO;
import blackboard.platform.plugin.PlugInUtil;
import blackboard.platform.security.authentication.BbSecurityException;
import blackboard.platform.servlet.InlineReceiptUtil;
import blackboard.platform.spring.beans.annotations.UserAuthorization;

// Denote this class as a Spring Controller
@Controller
public class ReviewController {
	private static final Logger LOGGER = Logger.getLogger(ReviewController.class.getName());
	// Autowire -- a.k.a. auto instantiate -- the DAOs for use in the controller
	@Autowired
	private UserDbLoader _userLoader;

	@Autowired
	private RubricDAO rubric_dao;

	@Autowired
	private RubricColumnDAO rubric_column_dao;

	@Autowired
	private RubricRowDAO rubric_row_dao;

	@Autowired
	private RubricRowEvalDAO rubric_row_eval_dao;

	@Autowired
	private RubricCellDAO rubric_cell_dao;

	@Autowired
	private AssessmentDAO assess_dao;

	@Autowired
	private ReviewOfEvaluationDAO review_dao;

	@Autowired
	private StudentReviewDAO student_review_dao;

	@Autowired
	private SubmissionDAO submission_dao;

	@Autowired
	private EvaluationDAO evaluation_dao;

	@Autowired
	private GradeDetailDAO grade_dao;

	@Autowired
	private GroupDbLoader _groupLoader;

	// Method ran when the students receives the list of submissions they are to
	// evaluate
	@RequestMapping("/review/student_reviews")
	public ModelAndView studentReviews(HttpServletRequest request, HttpServletResponse response,
			@RequestParam("assessment_id") int assessment_id) {
		ModelAndView mv = new ModelAndView("review/student_reviews");

		// Getting userID
		ContextManager contextManager = ContextManagerFactory.getInstance();
		Context ctx = contextManager.getContext();
		User user = ctx.getUser();
		int userIdInt = Integer.parseInt(new StringTokenizer(user.getId().toExternalString(), "_").nextToken());

		Assessment assessment = new Assessment();
		assessment = assess_dao.loadById(assessment_id);
		ArrayList<List> subsToEvaluate = new ArrayList<>();

		if (assessment.getAssignmentType().equals("G")) {
			try {
				List<Group> groupList = _groupLoader.loadByCourseId(ctx.getCourse().getId());
				
//				CourseMembershipDbLoader membershipLoader = null;
//				CourseMembership membership = null;
//
//				membershipLoader = CourseMembershipDbLoader.Default.getInstance();
//				membership = membershipLoader.loadByCourseAndUserId(ctx.getCourse().getId(), user.getId());

//				Group group = findGroup(assessment.getSubmissionGroups(), membership.getId());

				for (Group groupReview : groupList) {
//					if(groupReview.equals(group)){
//						continue;
//					}
					List submissions = new ArrayList<>();

					Submission sub = null;
					try {
						sub = submission_dao.searchByGroupIDAndAssessmentID(getDataTypeIdAsInt(Group.DATA_TYPE, groupReview.getId().getExternalString()), assessment_id);
					} catch (Exception e) {
						e.printStackTrace();
					}

					if (sub != null) {

						submissions.add(sub);
						if (assessment.getAnonymousGrading().equals("Y")) {
							submissions.add("Anonymous");
						} else {
							submissions.add(groupReview.getTitle());
						}

						submissions.add(false);
						submissions.add(false);
						
						subsToEvaluate.add(submissions);
					}
				}
			} catch (PersistenceException e1) {
				e1.printStackTrace();
			}
		} else {
			try {
				List<StudentReview> subsmissionsToEval = student_review_dao.searchByUserIDAndAssessmentID(userIdInt,
						assessment_id);
				for (StudentReview studReview : subsmissionsToEval) {
					List submissions = new ArrayList<>();
					LOGGER.debug("userToBeEval/SubmissionUser: " + studReview.getUserToBeEvalId() + " assessment_ID: "
							+ assessment_id);
					Submission sub = null;
					try {
						sub = submission_dao.searchByUserIDAndAssessmentID(studReview.getUserToBeEvalId(),
								assessment_id);
					} catch (Exception e) {
						e.printStackTrace();
					}

					Id userIdParam = Id.generateId(User.DATA_TYPE, studReview.getUserToBeEvalId());
					User userToEval = _userLoader.loadById(userIdParam);
					if (sub != null) {

						submissions.add(sub);
						if (assessment.getAnonymousGrading().equals("Y")) {
							submissions.add("Anonymous");
						} else {
							submissions.add(userToEval.getGivenName() + " " + userToEval.getFamilyName());
						}

						if (studReview.getIsReviewFinnished().equals("Y")) {
							submissions.add(true);
							Evaluation reviewEval = evaluation_dao.searchByUserIDAndSubmissionID(userIdInt,
									sub.getPropertyId());
							ReviewOfEvaluation reviewOfEvaluation = null;
							try {
								reviewOfEvaluation = review_dao.searchByEvaluationID(reviewEval.getPropertyId());
							} catch (KeyNotFoundException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							}
							if (reviewOfEvaluation == null) {
								submissions.add(false);
							} else {
								submissions.add(true);
							}

						} else {
							submissions.add(false);
							submissions.add(false);
						}
						subsToEvaluate.add(submissions);
					}
				}
			} catch (PersistenceException e1) {
				e1.printStackTrace();
			}
		}
		// for(List sub : subsToEvaluate){
		// LOGGER.debug("nameofeval1: " + sub.get(1));
		// }

		// // ModelAndView.addObject adds the objects you wish to pass to the
		// jsp
		mv.addObject("submissions", subsToEvaluate);
		mv.addObject("assessment", assessment);
		LOGGER.debug("review/student_reviews");

		// Returning the ModelAndView tells spring to find the JSP and set these
		// objects in the context for the request.
		return mv;
	}

	@RequestMapping("/review/redirectContent")
	private void redirectToContent(HttpServletRequest request, HttpServletResponse response) {
		final Context context = ContextManagerFactory.getInstance().getContext();
		final String contentReturnURL = PlugInUtil.getContentReturnURL(context.getContentId(), context.getCourseId());
		try {
			response.sendRedirect(contentReturnURL);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	// Method ran when the instructors enters the instructors view to view all
	// submitted submissions
	// @UserAuthorization("system.admin.VIEW")
	@RequestMapping("/review/instructor_reviews")
	public ModelAndView instructorReviews(HttpServletRequest request, HttpServletResponse response,
			@RequestParam("assessment_id") int assessment_id) {
		// Create a new ModelAndView Object. The constructor takes a string that
		// corresponds to the name of the jsp file the object should be routed
		// to
		ModelAndView mv = new ModelAndView("review/instructor_reviews");

		// Get all submission for selected Assessment
		List<Submission> submissions = new ArrayList<Submission>();
		try {
			submissions = submission_dao.findAllWithAssessmentID(assessment_id);
		} catch (KeyNotFoundException e) {
			LOGGER.debug("Assessment ID not found when searching for Submissions");
			e.printStackTrace();
		}

		// Get the assessment itself from id
		Assessment assessment = new Assessment();
		assessment = assess_dao.loadById(assessment_id);
		mv.addObject("assessment", assessment);

		// Group submission
		boolean isGroupSubmission = assessment.getAssignmentType().equals("G");
		mv.addObject("isGroupSubmission", isGroupSubmission);

		// Getting userID
		ContextManager contextManager = ContextManagerFactory.getInstance();
		Context ctx = contextManager.getContext();
		User user = ctx.getUser();
		int userIdInt = Integer.parseInt(new StringTokenizer(user.getId().toExternalString(), "_").nextToken());

		// Finds all evaluations with the current submission ID NOTE: this will
		// also find the currents user evaluation.
		ArrayList<ArrayList> submissionList = new ArrayList<>();
		for (int i = 0; i < submissions.size(); i++) {
			List<Evaluation> evaluations = new ArrayList<Evaluation>();
			try {
				evaluations = evaluation_dao.findAllWithSubmissiontID(submissions.get(i).getPropertyId());
			} catch (KeyNotFoundException e) {
				LOGGER.debug("Submission ID not found when searching for evaluations " + e);
			}

			ArrayList tempList = new ArrayList<>();
			tempList.add(submissions.get(i));
			int score = 0;
			for (int j = 0; j < evaluations.size(); j++) {
				score += evaluations.get(j).getEvaluationScore();
			}
			Integer userId = submissions.get(i).getSubmissionUserId();
			Id userIdParam;
			LOGGER.debug("Evaluations SIZE: " + evaluations.size() + "Evaluation Score: " + score);
			if (evaluations.size() > 0) {
				tempList.add((int) Math.ceil((double) score / evaluations.size()));
				LOGGER.debug("Evaluations SIZE ADDING: " + (int) Math.ceil((double) score / evaluations.size()));
			} else {
				tempList.add(0);
			}
			try {
				if (isGroupSubmission) {
					Id groupIdParam = Id.generateId(Group.DATA_TYPE,
							submissions.get(i).getGroupSubmissionId().toString());
					Group group = _groupLoader.loadById(groupIdParam);
					tempList.add(group);
				} else {
					userIdParam = Id.generateId(User.DATA_TYPE, userId.toString());
					User eval_user = _userLoader.loadById(userIdParam);
					tempList.add(eval_user.getGivenName() + " " + eval_user.getFamilyName());
				}
			} catch (PersistenceException e) {
				e.printStackTrace();
			}

			submissionList.add(tempList);
		}

		mv.addObject("submissions", submissionList);

		// Returning the ModelAndView tells spring to find the JSP and set these
		// objects in the context for the request.
		return mv;
	}

	// Method ran when an instructor or student presses "review or evaluate
	// assessment"
	@RequestMapping("/review/assessment_review")
	public ModelAndView assessmentReview(HttpServletRequest request, HttpServletResponse response,
			@RequestParam("assessment_id") int assessment_id, @RequestParam("course_id") String course_id,
			@RequestParam("submission_id") int submission_id) {
		// Create a new ModelAndView Object. The constructor takes a string that
		// corresponds to the name of the jsp file the object should be routed
		// to
		ModelAndView mv = new ModelAndView("review/assessment_review");

		// Get the assessment that is connected to the submission
		Assessment assessment = new Assessment();
		assessment = assess_dao.loadById(assessment_id);

		// Get selected submission
		Submission submission = new Submission();
		submission = submission_dao.loadById(submission_id);

		// Get all evaluations done on the submission
		List<Evaluation> evaluations = new ArrayList<Evaluation>();

		// Default value for the current users evaluation (this user)
		Integer emptyEvalID = null;
		mv.addObject("evaluation_id", emptyEvalID);
		mv.addObject("comment_text", "");
		mv.addObject("edit", false);
		mv.addObject("review_points_given", 0);
		mv.addObject("rubric_points_given", 0);
		mv.addObject("isOverrideRubricScore", false);
		mv.addObject("evaluation_id", -1);
		mv.addObject("feedbackText", "");
		mv.addObject("feedbackScore", 0);
		mv.addObject("isFeedbackEvaluationReceived", false);

		// Getting userID
		ContextManager contextManager = ContextManagerFactory.getInstance();
		Context ctx = contextManager.getContext();
		User user = ctx.getUser();

		// Check if instructor
		boolean instructor = false;
		LOGGER.debug("ROLE: " + user.getInstitutionRole());
		if (user.getInstitutionRole() != User.InstRole.STUDENT) {
			instructor = true;
			LOGGER.debug("ROLE: is true");
		}
		mv.addObject("isInstructor", instructor);
		User submissionUser = null;
		try {
			submissionUser = _userLoader
					.loadById(generateId(User.DATA_TYPE, Integer.toString(submission.getSubmissionUserId())));
		} catch (KeyNotFoundException e1) {
			e1.printStackTrace();
		} catch (PersistenceException e1) {
			e1.printStackTrace();
		}
		String username = submissionUser.getGivenName();
		if (assessment.getAnonymousGrading().equals("Y") && !instructor) {
			LOGGER.debug("setting to anonymous");
			mv.addObject("studentSubmissionName", "Anonymous");
		} else {
			mv.addObject("studentSubmissionName", username);
		}

		int userIdInt = Integer.parseInt(new StringTokenizer(user.getId().toExternalString(), "_").nextToken());

		// Group Evaluation
		int groupEvaluationId = -1;
		boolean isGroupEvaluation = assessment.getAssignmentType().equals("G");
		mv.addObject("isGroupEvaluation", isGroupEvaluation);

		if (isGroupEvaluation) {

			// First find the group that delivered the submission
			Group submissionGroup = null;
			try {
				submissionGroup = _groupLoader
						.loadById(generateId(Group.DATA_TYPE, Integer.toString(submission.getGroupSubmissionId())));
			} catch (KeyNotFoundException e) {
				e.printStackTrace();
			} catch (PersistenceException e) {
				e.printStackTrace();
			}
			mv.addObject("submissionGroup", submissionGroup);

			CourseMembershipDbLoader membershipLoader = null;
			CourseMembership membership = null;
			Id courseIdParam = generateId(Course.DATA_TYPE, course_id);

			try {
				membershipLoader = CourseMembershipDbLoader.Default.getInstance();
				membership = membershipLoader.loadByCourseAndUserId(courseIdParam, user.getId());
			} catch (KeyNotFoundException e1) {
				LOGGER.debug("Failed to get membership or loader: " + e1);
				e1.printStackTrace();
			} catch (PersistenceException e1) {
				LOGGER.debug("Failed to get membership or loader: " + e1);
				e1.printStackTrace();
			}

			// Finds the first group that the student is a member of
			Group group = findGroup(assessment.getSubmissionGroups(), membership.getId());

			if (group != null) {
				mv.addObject("group", group);
				groupEvaluationId = getDataTypeIdAsInt(Group.DATA_TYPE, group.getId().toExternalString());

			}
		}
		mv.addObject("groupEvalId", groupEvaluationId);

		Evaluation evaluation = null;
		if (isGroupEvaluation) {
			try {
				// Finds all evaluations with the current submission ID NOTE:
				// this will also find the currents user evaluation.
				evaluations = evaluation_dao.findAllWithSubmissiontID(submission_id);

				// Trying to find the current group evaluation.
				// If exsisting (e.g Evaluation has been created in database) it
				// sets edit to true and adds the correct evaluation id.
				evaluation = evaluation_dao.searchByGroupIDAndSubmissionID(groupEvaluationId, submission_id);
				if (evaluation != null) {
					mv.addObject("comment_text", FormattedText.toFormattedText(evaluation.getEvaluationText()));
					mv.addObject("edit", true);
					mv.addObject("evaluation_id", evaluation.getPropertyId());
					mv.addObject("review_points_given", evaluation.getOverrideEvaluationScore());
					mv.addObject("rubric_points_given", evaluation.getRubricEvaluationScore());
					if (evaluation.getOverrideRubricScore().equals("Y")) {
						mv.addObject("isOverrideRubricScore", true);
					} else {
						mv.addObject("isOverrideRubricScore", false);
					}
					LOGGER.debug("Evaluation found" + evaluation);
				}

			} catch (KeyNotFoundException e) {
				LOGGER.debug("Submission ID not found when searching for evaluations " + e);
				e.printStackTrace();
			}
		}
		if (evaluation == null) {
			try {
				// Finds all evaluations with the current submission ID NOTE:
				// this will also find the currents user evaluation.
				evaluations = evaluation_dao.findAllWithSubmissiontID(submission_id);

				// Trying to find the current users evaluation.
				// If exsisting (e.g Evaluation has been created in database) it
				// sets edit to true and adds the correct evaluation id.
				evaluation = evaluation_dao.searchByUserIDAndSubmissionID(userIdInt, submission_id);
				if (evaluation != null) {
					mv.addObject("comment_text", FormattedText.toFormattedText(evaluation.getEvaluationText()));
					mv.addObject("edit", true);
					mv.addObject("evaluation_id", evaluation.getPropertyId());
					mv.addObject("review_points_given", evaluation.getOverrideEvaluationScore());
					mv.addObject("rubric_points_given", evaluation.getRubricEvaluationScore());
					if (evaluation.getOverrideRubricScore().equals("Y")) {
						mv.addObject("isOverrideRubricScore", true);
					} else {
						mv.addObject("isOverrideRubricScore", false);
					}
					LOGGER.debug("Evaluation found" + evaluation);
				}

			} catch (KeyNotFoundException e) {
				LOGGER.debug("Submission ID not found when searching for evaluations " + e);
				e.printStackTrace();
			}
		}

		if (evaluation != null) {
			ReviewOfEvaluation reviewOfEvaluation = null;
			try {
				reviewOfEvaluation = review_dao.searchByEvaluationID(evaluation.getPropertyId());
			} catch (KeyNotFoundException e) {
				e.printStackTrace();
			}
			if (reviewOfEvaluation != null) {
				mv.addObject("feedbackText",
						reviewOfEvaluation.getReviewText().replaceAll("(\\r|\\n|\\r\\n)+", "<br>"));
				mv.addObject("feedbackScore", reviewOfEvaluation.getReviewScore());
				mv.addObject("isFeedbackEvaluationReceived", true);
			}
		}

		Rubric rubric = null;
		List<RubricRow> rubricRows = null;
		List<RubricColumn> rubricColumns = null;
		try {
			rubric = rubric_dao.findAllWithAssessmentID(assessment_id).get(0);
			rubricRows = rubric_row_dao.searchByRubricId(rubric.getPropertyId());
			rubricColumns = rubric_column_dao.searchByRubricId(rubric.getPropertyId());
			LOGGER.debug("rubric: " + rubric);
			LOGGER.debug("rubricRows: " + rubricRows);
			LOGGER.debug("rubricColumns: " + rubricColumns);
		} catch (KeyNotFoundException e1) {
			e1.printStackTrace();
		} catch (NumberFormatException e1) {
			e1.printStackTrace();
		}
		rubricColumns.subList(0, rubric.getNumberOfScales() + 1);

		int counter = 0;
		ArrayList<ArrayList<ArrayList<String>>> cells = new ArrayList();
		ArrayList<ArrayList<String>> cellsPerRow = new ArrayList<>();
		for (int i = 0; i < rubricRows.size(); i++) {
			for (int j = 0; j < rubricColumns.size(); j++) {
				ArrayList<String> arrayCell = new ArrayList<>();
				RubricCell cell = rubric_cell_dao.searchByColAndRowId(rubricColumns.get(j).getPropertyId(),
						rubricRows.get(i).getPropertyId());
				LOGGER.debug("RubricCell: " + cell);
				arrayCell.add(cell.getPoints() + "");
				if(cell.getDescription() == null){
					arrayCell.add("");
				}else{
					arrayCell.add(
							"\"" + cell.getDescription().replaceAll("(\\r|\\n|\\r\\n)+", "<br>").replaceAll("\"", "\\\\\"")
									+ "\"");
				}
				cellsPerRow.add(arrayCell);
			}
			cells.add(cellsPerRow);
			cellsPerRow = new ArrayList<>();

		}

		// // ModelAndView.addObject adds the objects you wish to pass to the
		// jsp
		mv.addObject("assessment_id", assessment_id);
		mv.addObject("assessment", assessment);
		mv.addObject("submission_id", submission_id);
		mv.addObject("max_points", assessment.getPointsPossible());
		mv.addObject("submission", submission);
		mv.addObject("cellsInRows", cells);
		mv.addObject("numberOfScales", rubric.getNumberOfScales());
		mv.addObject("scalePosCreated", false);
		mv.addObject("scalePos", 1);

		if (evaluation != null && rubric != null) {
			ArrayList<Integer> scalePos = new ArrayList<>();
			try {
				List<RubricRowEval> evalRows = rubric_row_eval_dao.searchByEvalIdAndRubricId(evaluation.getPropertyId(),
						rubric.getPropertyId());
				Collections.sort(evalRows, new Comparator<RubricRowEval>() {
					@Override
					public int compare(RubricRowEval o1, RubricRowEval o2) {
						return o1.getRowPosition().compareTo(o2.getRowPosition());
					}
				});
				for (RubricRowEval rowEval : evalRows) {
					LOGGER.debug("rowEval_test" + rowEval);
					LOGGER.debug("chosenScale " + rowEval.getChosenScale());
					LOGGER.debug("rowEvalPropID " + rowEval.getPropertyId());
					scalePos.add(rowEval.getChosenScale());
				}
				mv.addObject("scalePos", scalePos);
				mv.addObject("scalePosCreated", true);
			} catch (KeyNotFoundException e) {
				e.printStackTrace();
			}
		}

		ArrayList<ArrayList<ArrayList>> evalsWithName = new ArrayList<>();
		for (int i = 0; i < evaluations.size(); i++) {
			ArrayList<Integer> scalePos = new ArrayList<>();
			Integer userId = evaluations.get(i).getReviewerUserId();
			if (evaluation != null && userId == evaluation.getReviewerUserId()) {
				continue;
			}

			Id userIdParam;
			try {
				userIdParam = Id.generateId(User.DATA_TYPE, userId);
				User eval_user = _userLoader.loadById(userIdParam);
				ArrayList tempList = new ArrayList<>();
				if (isGroupEvaluation) {
					Integer groupId = evaluations.get(i).getGroupEvaluationId();
					Id groupIdParam = Id.generateId(Group.DATA_TYPE, Integer.toString(groupId));
					Group eval_group = _groupLoader.loadById(groupIdParam);
					LOGGER.debug("isGroup Title: " + eval_group + " TILE: " + eval_group.getTitle());
					tempList.add('"' + eval_group.getTitle() + '"');
				} else {
					tempList.add('"' + eval_user.getGivenName() + " " + eval_user.getFamilyName() + '"');
				}
				List<RubricRowEval> evalRows = rubric_row_eval_dao
						.searchByEvalIdAndRubricId(evaluations.get(i).getPropertyId(), rubric.getPropertyId());
				Collections.sort(evalRows, new Comparator<RubricRowEval>() {
					@Override
					public int compare(RubricRowEval o1, RubricRowEval o2) {
						return o1.getRowPosition().compareTo(o2.getRowPosition());
					}
				});
				for (RubricRowEval rowEval : evalRows) {
					scalePos.add(rowEval.getChosenScale());
				}
				ArrayList evaluationInStrings = new ArrayList<>();
				evaluationInStrings.add(evaluations.get(i).getPropertyId());
				evaluationInStrings.add(evaluations.get(i).getEvaluationScore());
				evaluationInStrings.add(evaluations.get(i).getRubricEvaluationScore());
				evaluationInStrings.add('\"' + evaluations.get(i).getEvaluationText() + '\"');
				evaluationInStrings.add(evaluations.get(i).getOverrideEvaluationScore());
				evaluationInStrings.add('\"' + evaluations.get(i).getOverrideRubricScore() + '\"');
				evaluationInStrings.add(scalePos);
				tempList.add(evaluationInStrings);
				evalsWithName.add(tempList);

			} catch (PersistenceException e) {
				e.printStackTrace();
			}

		}

		mv.addObject("evaluations", evalsWithName);
		if (evalsWithName.size() == 0) {
			mv.addObject("isOtherReviews", false);
		} else {
			mv.addObject("isOtherReviews", true);
		}

		CourseFileManager crsFileMng = new CourseFileManager();
		mv.addObject("filePath", "_no path_");
		mv.addObject("currentFileName", "No File");
		mv.addObject("currentFile", "No File");
		if (submission != null) {
			mv.addObject("currentFileName", "No File [0]");
			try {
				File root = crsFileMng
						.getRootDirectory(Integer.toString(getDataTypeIdAsInt(Course.DATA_TYPE, course_id)));

				Path path = Paths.get(root.getPath() + "/b2_peer_assessment/assessment_id_"
						+ Integer.toString(getDataTypeIdAsInt(Assessment.DATA_TYPE, Integer.toString(assessment_id)))
						+ "/submissions_files/pk1_" + submission.getPropertyId());
				mv.addObject("currentFileName", "No File [1]");
				mv.addObject("filePath", path);

				File submittedfile = new File(path.toString());

				if (Files.exists(path)) {
					mv.addObject("currentFileName", "File exsist [2]");
					mv.addObject("currentFileName", submittedfile.getName());

				}
			} catch (FileSystemException e) {
				LOGGER.debug("FAILED TO GET ROOT WHEN LOADING SUBMISSION: " + e);
				e.printStackTrace();
			}
		}

		// Returning the ModelAndView tells spring to find the JSP and set these
		// objects in the context for the request.
		return mv;
	}

	// Method ran when student pressess submission after submission time has
	// ended.
	// It fetches the submission, evaluation and reivew of the evaluation
	@RequestMapping("/review/reviewOfReview")
	public ModelAndView review(HttpServletRequest request, HttpServletResponse response,
			@RequestParam("content_id") String content_id, @RequestParam("course_id") String course_id,
			@RequestParam("assessment_id") int assessment_id, @RequestParam("submission_id") int submission_id) {

		ModelAndView mv = new ModelAndView("/review/review_of_review");

		LOGGER.debug("reviewOfReview started. SubmissionID: " + submission_id);

		// Runs a check if submission id is -1 then there is no submission and
		// no code is needed.
		// The user will be displayed a text saying that "No submission found"
		// and nothing more.
		boolean isSubmissionExsisting = true;
		if (submission_id == -1) {
			isSubmissionExsisting = false;
			mv.addObject("isSubmissionExsisting", isSubmissionExsisting);
			return mv;
		}
		mv.addObject("isSubmissionExsisting", isSubmissionExsisting);

		// Get the assessment that is connected to the review
		Assessment assessment = new Assessment();
		assessment = assess_dao.loadById(assessment_id);

		// Get selected submission
		Submission submission = new Submission();
		submission = submission_dao.loadById(submission_id);

		// Get all evaluations done on the submission
		List<Evaluation> evaluations = new ArrayList<Evaluation>();

		// Default value for the current users evaluation (this user)
		Integer emptyEvalID = null;
		mv.addObject("evaluation_id", emptyEvalID);
		mv.addObject("rubric_points_given", 0);
		mv.addObject("isOverrideRubricScore", false);
		mv.addObject("evaluation_id", -1);
		mv.addObject("review_of_review_max_score", assessment.getReviewOfReviewMaxScore());

		// Getting userID
		ContextManager contextManager = ContextManagerFactory.getInstance();
		Context ctx = contextManager.getContext();
		User user = ctx.getUser();

		User submissionUser = null;
		try {
			submissionUser = _userLoader
					.loadById(generateId(User.DATA_TYPE, Integer.toString(submission.getSubmissionUserId())));
		} catch (KeyNotFoundException e1) {
			e1.printStackTrace();
		} catch (PersistenceException e1) {
			e1.printStackTrace();
		}
		String username = submissionUser.getGivenName();
		mv.addObject("studentSubmissionName", username);

		int userIdInt = Integer.parseInt(new StringTokenizer(user.getId().toExternalString(), "_").nextToken());

		// Group Evaluation
		int groupEvaluationId = -1;
		boolean isGroupEvaluation = assessment.getAssignmentType().equals("G");
		mv.addObject("isGroupEvaluation", isGroupEvaluation);

		if (isGroupEvaluation) {

			// First find the group that delivered the submission
			Group submissionGroup = null;
			try {
				submissionGroup = _groupLoader
						.loadById(generateId(Group.DATA_TYPE, Integer.toString(submission.getGroupSubmissionId())));
			} catch (KeyNotFoundException e) {
				e.printStackTrace();
			} catch (PersistenceException e) {
				e.printStackTrace();
			}
			mv.addObject("submissionGroup", submissionGroup);

			CourseMembershipDbLoader membershipLoader = null;
			CourseMembership membership = null;
			Id courseIdParam = generateId(Course.DATA_TYPE, course_id);

			try {
				membershipLoader = CourseMembershipDbLoader.Default.getInstance();
				membership = membershipLoader.loadByCourseAndUserId(courseIdParam, user.getId());
			} catch (KeyNotFoundException e1) {
				LOGGER.debug("Failed to get membership or loader: " + e1);
				e1.printStackTrace();
			} catch (PersistenceException e1) {
				LOGGER.debug("Failed to get membership or loader: " + e1);
				e1.printStackTrace();
			}

			// Finds the first group that the student is a member of
			Group group = findGroup(assessment.getSubmissionGroups(), membership.getId());

			if (group != null) {
				mv.addObject("group", group);
				groupEvaluationId = getDataTypeIdAsInt(Group.DATA_TYPE, group.getId().toExternalString());

			}
		}
		mv.addObject("groupEvalId", groupEvaluationId);

		Evaluation evaluation = null;
		if (isGroupEvaluation) {
			try {
				// Finds all evaluations with the current submission ID NOTE:
				// this will also find the currents user evaluation.
				evaluations = evaluation_dao.findAllWithSubmissiontID(submission_id);

				// Trying to find the current group evaluation.
				// If exsisting (e.g Evaluation has been created in database) it
				// sets edit to true and adds the correct evaluation id.
				evaluation = evaluation_dao.searchByGroupIDAndSubmissionID(groupEvaluationId, submission_id);
				if (evaluation != null) {
					mv.addObject("comment_text", FormattedText.toFormattedText(evaluation.getEvaluationText()));
					mv.addObject("evaluation_id", evaluation.getPropertyId());
					mv.addObject("review_points_given", evaluation.getOverrideEvaluationScore());
					mv.addObject("rubric_points_given", evaluation.getRubricEvaluationScore());
					if (evaluation.getOverrideRubricScore().equals("Y")) {
						mv.addObject("isOverrideRubricScore", true);
					} else {
						mv.addObject("isOverrideRubricScore", false);
					}
					LOGGER.debug("Evaluation found" + evaluation);
				}

			} catch (KeyNotFoundException e) {
				LOGGER.debug("Submission ID not found when searching for evaluations " + e);
				e.printStackTrace();
			}
		}
		if (evaluation == null) {
			try {
				// Finds all evaluations with the current submission ID NOTE:
				// this will also find the currents user evaluation.
				evaluations = evaluation_dao.findAllWithSubmissiontID(submission_id);

				// Trying to find the current users evaluation.
				// If exsisting (e.g Evaluation has been created in database) it
				// sets edit to true and adds the correct evaluation id.
				evaluation = evaluation_dao.searchByUserIDAndSubmissionID(userIdInt, submission_id);
				if (evaluation != null) {
					mv.addObject("comment_text", FormattedText.toFormattedText(evaluation.getEvaluationText()));
					mv.addObject("evaluation_id", evaluation.getPropertyId());
					mv.addObject("review_points_given", evaluation.getOverrideEvaluationScore());
					mv.addObject("rubric_points_given", evaluation.getRubricEvaluationScore());
					if (evaluation.getOverrideRubricScore().equals("Y")) {
						mv.addObject("isOverrideRubricScore", true);
					} else {
						mv.addObject("isOverrideRubricScore", false);
					}
					LOGGER.debug("Evaluation found" + evaluation);
				}

			} catch (KeyNotFoundException e) {
				LOGGER.debug("Submission ID not found when searching for evaluations " + e);
				e.printStackTrace();
			}
		}

		Rubric rubric = null;
		List<RubricRow> rubricRows = null;
		List<RubricColumn> rubricColumns = null;
		try {
			rubric = rubric_dao.findAllWithAssessmentID(assessment_id).get(0);
			rubricRows = rubric_row_dao.searchByRubricId(rubric.getPropertyId());
			rubricColumns = rubric_column_dao.searchByRubricId(rubric.getPropertyId());
		} catch (KeyNotFoundException e1) {
			e1.printStackTrace();
		} catch (NumberFormatException e1) {
			e1.printStackTrace();
		}
		rubricColumns.subList(0, rubric.getNumberOfScales() + 1);

		int counter = 0;
		ArrayList<ArrayList<ArrayList<String>>> cells = new ArrayList();
		ArrayList<ArrayList<String>> cellsPerRow = new ArrayList<>();
		for (int i = 0; i < rubricRows.size(); i++) {
			for (int j = 0; j < rubricColumns.size(); j++) {
				ArrayList<String> arrayCell = new ArrayList<>();
				RubricCell cell = rubric_cell_dao.searchByColAndRowId(rubricColumns.get(j).getPropertyId(),
						rubricRows.get(i).getPropertyId());
				arrayCell.add(cell.getPoints() + "");
				arrayCell.add(
						"\"" + cell.getDescription().replaceAll("(\\r|\\n|\\r\\n)+", "<br>").replaceAll("\"", "\\\\\"")
								+ "\"");
				cellsPerRow.add(arrayCell);
			}
			cells.add(cellsPerRow);
			cellsPerRow = new ArrayList<>();

		}

		// ModelAndView.addObject adds the objects you wish to pass to the jsp
		mv.addObject("assessment_id", assessment_id);
		mv.addObject("assessment", assessment);
		mv.addObject("submission_id", submission_id);
		mv.addObject("content_id", content_id);
		mv.addObject("max_points", assessment.getPointsPossible());
		mv.addObject("submission", submission);
		mv.addObject("cellsInRows", cells);
		mv.addObject("numberOfScales", rubric.getNumberOfScales());
		mv.addObject("scalePosCreated", false);
		mv.addObject("scalePos", 1);

		if (evaluation != null && rubric != null) {
			ArrayList<Integer> scalePos = new ArrayList<>();
			try {
				List<RubricRowEval> evalRows = rubric_row_eval_dao.searchByEvalIdAndRubricId(evaluation.getPropertyId(),
						rubric.getPropertyId());
				Collections.sort(evalRows, new Comparator<RubricRowEval>() {
					@Override
					public int compare(RubricRowEval o1, RubricRowEval o2) {
						return o1.getRowPosition().compareTo(o2.getRowPosition());
					}
				});
				for (RubricRowEval rowEval : evalRows) {
					LOGGER.debug("rowEval_test" + rowEval);
					LOGGER.debug("chosenScale " + rowEval.getChosenScale());
					LOGGER.debug("rowEvalPropID " + rowEval.getPropertyId());
					scalePos.add(rowEval.getChosenScale());
				}
				mv.addObject("scalePos", scalePos);
				mv.addObject("scalePosCreated", true);
			} catch (KeyNotFoundException e) {
				e.printStackTrace();
			}
		}

		ArrayList<ArrayList> evalsWithName = new ArrayList<>();
		ArrayList<FormattedText> feedbackTextbox = new ArrayList<>();
		ArrayList<String> evaluationText = new ArrayList<>();
		for (int i = 0; i < evaluations.size(); i++) {
			ArrayList<Integer> scalePos = new ArrayList<>();
			Integer userId = evaluations.get(i).getReviewerUserId();
			Id userIdParam;

			// See if the evaluation already has a reivew
			ReviewOfEvaluation review = null;
			try {
				review = review_dao.searchByEvaluationID(evaluations.get(i).getPropertyId());
			} catch (KeyNotFoundException e1) {
				e1.printStackTrace();
			}

			try {
				userIdParam = Id.generateId(User.DATA_TYPE, userId);
				User eval_user = _userLoader.loadById(userIdParam);
				ArrayList tempList = new ArrayList<>();
				if (isGroupEvaluation) {
					Integer groupId = evaluations.get(i).getGroupEvaluationId();
					Id groupIdParam = Id.generateId(Group.DATA_TYPE, Integer.toString(groupId));
					Group eval_group = _groupLoader.loadById(groupIdParam);
					LOGGER.debug("isGroup Title: " + eval_group + " TILE: " + eval_group.getTitle());
					if (assessment.getAnonymousGrading().equals("Y")) {
						tempList.add('"' + "Anonymous" + '"');
					} else {
						tempList.add('"' + eval_group.getTitle() + '"');
					}
				} else {
					if (assessment.getAnonymousGrading().equals("Y")) {
						tempList.add('"' + "Anonymous" + '"');
					} else {
						tempList.add('"' + eval_user.getGivenName() + " " + eval_user.getFamilyName() + '"');
					}
				}
				List<RubricRowEval> evalRows = rubric_row_eval_dao
						.searchByEvalIdAndRubricId(evaluations.get(i).getPropertyId(), rubric.getPropertyId());
				Collections.sort(evalRows, new Comparator<RubricRowEval>() {
					@Override
					public int compare(RubricRowEval o1, RubricRowEval o2) {
						return o1.getRowPosition().compareTo(o2.getRowPosition());
					}
				});
				for (RubricRowEval rowEval : evalRows) {
					scalePos.add(rowEval.getChosenScale());
				}
				ArrayList evaluationInStrings = new ArrayList<>();
				evaluationInStrings.add(evaluations.get(i).getPropertyId());
				evaluationInStrings.add(evaluations.get(i).getEvaluationScore());
				evaluationInStrings.add(evaluations.get(i).getRubricEvaluationScore());
				evaluationInStrings.add(-1);
				evaluationInStrings.add(evaluations.get(i).getOverrideEvaluationScore());
				evaluationInStrings.add('\"' + evaluations.get(i).getOverrideRubricScore() + '\"');
				evaluationInStrings.add(scalePos);
				tempList.add(evaluationInStrings);
				evaluationText.add(evaluations.get(i).getEvaluationText());
				if (review != null) {
					// tempList.add(review.getReviewText());
					feedbackTextbox.add(FormattedText.toFormattedText(review.getReviewText()));
					tempList.add(i);
					tempList.add(review.getReviewScore());
				} else {
					// tempList.add("");
					feedbackTextbox.add(FormattedText.toFormattedText(""));
					tempList.add(i);
					tempList.add(0);
				}
				evalsWithName.add(tempList);

			} catch (PersistenceException e) {
				e.printStackTrace();
			}

		}

		mv.addObject("evaluations", evalsWithName);
		mv.addObject("feedbackTextbox", feedbackTextbox);
		mv.addObject("evaluationText", evaluationText);

		CourseFileManager crsFileMng = new CourseFileManager();
		mv.addObject("filePath", "_no path_");
		mv.addObject("currentFileName", "No File");
		mv.addObject("currentFile", "No File");
		if (submission != null) {
			mv.addObject("currentFileName", "No File [0]");
			try {
				File root = crsFileMng
						.getRootDirectory(Integer.toString(getDataTypeIdAsInt(Course.DATA_TYPE, course_id)));

				Path path = Paths.get(root.getPath() + "/b2_peer_assessment/assessment_id_"
						+ Integer.toString(getDataTypeIdAsInt(Assessment.DATA_TYPE, Integer.toString(assessment_id)))
						+ "/submissions_files/pk1_" + submission.getPropertyId());
				mv.addObject("currentFileName", "No File [1]");
				mv.addObject("filePath", path);

				File submittedfile = new File(path.toString());

				if (Files.exists(path)) {
					mv.addObject("currentFileName", "File exsist [2]");
					mv.addObject("currentFileName", submittedfile.getName());

				}
			} catch (FileSystemException e) {
				LOGGER.debug("FAILED TO GET ROOT WHEN LOADING SUBMISSION: " + e);
				e.printStackTrace();
			}
		}

		LOGGER.debug("review/reviewOfReview done");

		return mv;
	}

	// Method ran when user presses submit on the review of review page
	@RequestMapping("/review/saveReviewOfReview")
	private void saveReviewOfReview(HttpServletRequest request, HttpServletResponse response,
			@RequestParam("content_id") String contentId, @RequestParam("course_id") String courseId,
			@RequestParam("assessment_id") int assessment_id, @RequestParam("submission_id") int submission_id,
			@RequestParam("evaluation_id") int[] evaluation_id,
			@RequestParam("review_of_review_score") int[] review_of_review_score,
			@RequestParam("feedback_textboxtext") String[] feedback_textbox) {

		for (int i = 0; i < evaluation_id.length; i++) {
			int evalId = evaluation_id[i];
			String reviewText = "";
			if (feedback_textbox.length > 0) {
				reviewText = feedback_textbox[i];
			}
			int reviewScore = review_of_review_score[i];
			ReviewOfEvaluation reviewOfEvaluation = null;
			try {
				reviewOfEvaluation = review_dao.searchByEvaluationID(evalId);
			} catch (KeyNotFoundException e) {
				e.printStackTrace();
			}
			if (reviewOfEvaluation == null) {
				reviewOfEvaluation = new ReviewOfEvaluation();
				reviewOfEvaluation.setPeerEvaluationId(evalId);
			}
			reviewOfEvaluation.setReviewText(reviewText);
			reviewOfEvaluation.setReviewScore(reviewScore);
			review_dao.save(reviewOfEvaluation);
		}

		try {
			ReceiptOptions ro = new ReceiptOptions();
			ro.addSuccessMessage("Properties Successfully Updated.");
			InlineReceiptUtil.addReceiptToRequest(request, ro);

			response.sendRedirect("reviewOfReview?course_id=" + courseId + "&content_id=" + contentId + contentId
					+ "&assessment_id=" + assessment_id + "&submission_id=" + submission_id);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	// Method ran when the evaluator pressess submit on an evaluation of a
	// submission.
	@RequestMapping("/review/saveEvaluation")
	private void saveEvaluation(HttpServletRequest request, HttpServletResponse response,
			@RequestParam("assessment_id") int assessment_id, @RequestParam("submission_id") int submission_id,
			@RequestParam("evaluation_id") Integer evaluation_id,
			@RequestParam("comment_textboxtext") String comment_textbox, @RequestParam("user_id") String userId,
			@RequestParam("course_id") String course_id, @RequestParam("edit") boolean edit,
			@RequestParam("review_score") int review_score,
			@RequestParam(required = false, value = "override_rubric_score") String overrideRubricScore,
			@RequestParam("points_given") String pointsGiven, @RequestParam("userToBeEval_id") int userToEvalId,
			@RequestParam("group_evaluation_id") Integer groupEvaluationId,
			@RequestParam("scale_pos_input") String[] scalePosInput) {

		Evaluation evaluation;
		if (edit) {
			evaluation = evaluation_dao.loadById(evaluation_id);
		} else {
			evaluation = new Evaluation();
		}

		// set all prefs to incoming request parameter values
		evaluation.setReviewerUserId(getUserIdAsInt(userId));
		evaluation.setPeerSubmissiontId(submission_id);
		evaluation.setEvaluationText(comment_textbox);
		evaluation.setOverrideEvaluationScore(review_score);
		evaluation.setRubricEvaluationScore(Integer.parseInt(pointsGiven));
		evaluation.setGroupEvaluationId(groupEvaluationId);

		if (overrideRubricScore == null) {
			evaluation.setOverrideRubricScore("N");
			evaluation.setEvaluationScore(Integer.parseInt(pointsGiven));
		} else {
			evaluation.setOverrideRubricScore("Y");
			evaluation.setEvaluationScore(review_score);

		}

		// save the prefs
		evaluation_dao.save(evaluation);

		try {
			StudentReview studRev = student_review_dao
					.searchByEvaluatorAndUserToEvaluateId(getUserIdAsInt(userId), userToEvalId, assessment_id)
					.setIsReviewFinnished("Y");
			LOGGER.debug("studReview: " + studRev);
			student_review_dao.save(studRev);
		} catch (KeyNotFoundException e2) {
			e2.printStackTrace();
		} catch (NumberFormatException e2) {
			e2.printStackTrace();
		}

		try {
			int newEvaluationId = evaluation_dao.searchByUserIDAndSubmissionID(getUserIdAsInt(userId), submission_id)
					.getPropertyId();
			Rubric rubric = rubric_dao.findAllWithAssessmentID(assessment_id).get(0);
			int counter = 1;
			int row = 0;
			RubricRowEval rubricRowEval;
			if (edit) {
				LOGGER.debug("edit is true ");
				rubricRowEval = rubric_row_eval_dao.searchByEvalIdAndRubricIdandPos(newEvaluationId,
						rubric.getPropertyId(), row);
				LOGGER.debug("rubricRowEvalROW: " + row);
			} else {
				rubricRowEval = new RubricRowEval();
				rubricRowEval.setEvaluationId(newEvaluationId);
				rubricRowEval.setRubricId(rubric.getPropertyId());
				rubricRowEval.setRowPosition(row);
			}

			for (int i = 0; i < scalePosInput.length; i++) {
				LOGGER.debug("substringscale " + scalePosInput[i].substring(2));
				if (scalePosInput[i].substring(2).equalsIgnoreCase("checked")) {
					rubricRowEval.setChosenScale(counter);
				}
				if (counter == rubric.getNumberOfScales()) {
					if (!edit) {
						rubric_row_eval_dao
								.save(new RubricRowEval(rubricRowEval.getRubricId(), rubricRowEval.getEvaluationId(),
										rubricRowEval.getChosenScale(), rubricRowEval.getRowPosition()));
					} else {
						rubric_row_eval_dao.save(rubricRowEval);
					}

					row++;
					if (i != scalePosInput.length - 1) {
						if (edit) {
							rubricRowEval = rubric_row_eval_dao.searchByEvalIdAndRubricIdandPos(newEvaluationId,
									rubric.getPropertyId(), row);
							LOGGER.debug("rubricRowEvalROW: " + row);
						} else {
							rubricRowEval = new RubricRowEval();
							rubricRowEval.setEvaluationId(newEvaluationId);
							rubricRowEval.setRubricId(rubric.getPropertyId());
							LOGGER.debug("rubricRowEvalROWnotedit: " + row);
							rubricRowEval.setRowPosition(row);
						}
					}
					counter = 0;
				}
				counter++;
			}

		} catch (KeyNotFoundException e1) {
			e1.printStackTrace();
		}

		try {
			ReceiptOptions ro = new ReceiptOptions();
			ro.addSuccessMessage("Properties Successfully Updated.");
			InlineReceiptUtil.addReceiptToRequest(request, ro);

			response.sendRedirect("assessment_review?course_id=" + course_id + "&assessment_id=" + assessment_id
					+ "&submission_id=" + submission_id);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Submitts a score based on studentId into the given GradableItem.
	 * 
	 * GradableItem: there is one GradableItem for the whole assessment. One
	 * GradableItem has all students scores for that particular assessment. The
	 * item is based on contentId and if no item exsists one is created.
	 * 
	 * @param courseId
	 * @param contentId
	 */
	/*
	 * @RequestMapping(value = "/review/submitScore", method =
	 * RequestMethod.POST)
	 * 
	 * @ResponseStatus(value = HttpStatus.OK)
	 */
	@UserAuthorization("system.admin.VIEW")
	@RequestMapping("/review/submitScore")
	private void submitScore(HttpServletRequest request, HttpServletResponse response,
			@RequestParam("assessment_id") int assessment_id, @RequestParam("course_id") String course_id,
			@RequestParam("content_id") String content_id, @RequestParam("user_id") String user_id,
			@RequestParam(value = "group_id", required = false) String group_id,
			@RequestParam("final_score") String final_score) {

		// Making string and ints into IDs:
		Id courseIdParam = generateId(Course.DATA_TYPE, course_id);
		Id contentIdParam = generateId(Content.DATA_TYPE, content_id);
		Id userIdParam = generateId(Content.DATA_TYPE, user_id);

		Id groupIdParam = null;
		boolean isGroupSubmission = (group_id != null);
		if (isGroupSubmission) {
			groupIdParam = generateId(Group.DATA_TYPE, group_id);
		}

		Assessment assessment = new Assessment();
		assessment = assess_dao.loadById(assessment_id);
		LOGGER.debug("submitScore Assessment: " + assessment);
		LOGGER.debug("Values assess Id: " + assessment_id + " cId: " + course_id + " contId " + content_id);

		GradableItemManager giManager = GradebookManagerFactory.getGradableItemManager();

		/**
		 * SETTING STUDENT GRADE Getting gradable item again to get propper ID)
		 */
		GradableItem column = null;
		try {
			column = giManager.getGradebookItemByContentId(contentIdParam);
		} catch (PersistenceRuntimeException e1) {
			LOGGER.debug("Failed to get gradable item: " + e1);
			e1.printStackTrace();
		} catch (BbSecurityException e1) {
			LOGGER.debug("Failed to get gradable item: " + e1);
			e1.printStackTrace();
		}
		CourseMembershipDbLoader membershipLoader = null;

		if (isGroupSubmission) {
			LOGGER.debug("Group submission! GroupID:" + groupIdParam);
			List<CourseMembership> memberships = new ArrayList<CourseMembership>();
			try {
				Group group = _groupLoader.loadById(groupIdParam);
				membershipLoader = CourseMembershipDbLoader.Default.getInstance();
				for (GroupMembership groupMembership : group.getGroupMemberships()) {
					CourseMembership membership = membershipLoader.loadById(groupMembership.getCourseMembershipId());
					memberships.add(membership);
				}

			} catch (KeyNotFoundException e1) {
				LOGGER.debug("Failed to get membership or loader: " + e1);
				e1.printStackTrace();
			} catch (PersistenceException e1) {
				LOGGER.debug("Failed to get membership or loader: " + e1);
				e1.printStackTrace();
			}
			LOGGER.debug("Group submission! memberships:" + memberships + " size: " + memberships.size());

			GradeDetailDAO gradeDetailDAO = grade_dao;
			AssessmentDAO assessmentDAO = assess_dao;

			// GradableItem column
			for (CourseMembership courseMembership : memberships) {

				GradeDetail userGrade = gradeDetailDAO.getGradeDetail(column.getId(), courseMembership.getId());

				if (userGrade == null) {
					Id newUserGradeId = gradeDetailDAO.createNullGrade(courseMembership.getId(), column.getId());
					userGrade = new GradeDetail();
					userGrade.setId(newUserGradeId);
					userGrade.setCourseUserId(courseMembership.getId());
					userGrade.setGradableItem(column);
				}

				userGrade.setManualScore(Double.parseDouble(final_score));
				userGrade.setManualGrade(final_score);

				gradeDetailDAO.persist(userGrade);
			}

			/**
			 * Updates the submission with final score
			 */
			// Get selected submission
			Submission submission = new Submission();
			try {
				submission = submission_dao.searchByGroupIDAndAssessmentID(getUserIdAsInt(group_id), assessment_id);
				submission.setFinalScore(Integer.parseInt(final_score));
				submission_dao.persist(submission);
			} catch (KeyNotFoundException e1) {
				LOGGER.debug("Failed to load or set submission final score " + e1);
				e1.printStackTrace();
			}
		} else {
			CourseMembership membership = null;

			try {
				membershipLoader = CourseMembershipDbLoader.Default.getInstance();
				membership = membershipLoader.loadByCourseAndUserId(courseIdParam, userIdParam);
			} catch (KeyNotFoundException e1) {
				LOGGER.debug("Failed to get membership or loader: " + e1);
				e1.printStackTrace();
			} catch (PersistenceException e1) {
				LOGGER.debug("Failed to get membership or loader: " + e1);
				e1.printStackTrace();
			}

			GradeDetailDAO gradeDetailDAO = grade_dao;
			AssessmentDAO assessmentDAO = assess_dao;

			// GradableItem column
			GradeDetail userGrade = gradeDetailDAO.getGradeDetail(column.getId(), membership.getId());

			if (userGrade == null) {
				Id newUserGradeId = gradeDetailDAO.createNullGrade(membership.getId(), column.getId());
				userGrade = new GradeDetail();
				userGrade.setId(newUserGradeId);
				userGrade.setCourseUserId(membership.getId());
				userGrade.setGradableItem(column);
			}

			userGrade.setManualScore(Double.parseDouble(final_score));
			userGrade.setManualGrade(final_score);

			gradeDetailDAO.persist(userGrade);

			/**
			 * Updates the submission with final score
			 */
			// Get selected submission
			Submission submission = new Submission();
			try {
				submission = submission_dao.searchByUserIDAndAssessmentID(getUserIdAsInt(user_id), assessment_id);
				submission.setFinalScore(Integer.parseInt(final_score));
				submission_dao.persist(submission);
			} catch (KeyNotFoundException e1) {
				LOGGER.debug("Failed to load or set submission final score " + e1);
				e1.printStackTrace();
			}
		}
		try {
			ReceiptOptions ro = new ReceiptOptions();
			ro.addSuccessMessage("Submission Successfully Updated.");
			InlineReceiptUtil.addReceiptToRequest(request, ro);

			response.sendRedirect("instructor_reviews?course_id=" + course_id + "&assessment_id=" + assessment_id
					+ "&content_id=" + content_id);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@RequestMapping(value = "/fetchFile")
	public void getFile(@RequestParam("file_name") String fileName, @RequestParam("assessment_id") String assessmentId,
			@RequestParam("submission_id") String submissionId, @RequestParam("course_id") String courseId,
			HttpServletResponse response) {
		CourseFileManager crsFileMng = new CourseFileManager();
		File root;
		try {
			// get your file as InputStream
			root = crsFileMng.getRootDirectory(Integer.toString(getDataTypeIdAsInt(Course.DATA_TYPE, courseId)));
			Path path = Paths.get(root.getPath() + "/b2_peer_assessment/assessment_id_"
					+ Integer.toString(getDataTypeIdAsInt(Assessment.DATA_TYPE, assessmentId))
					+ "/submissions_files/pk1_" + submissionId);

			InputStream is = new FileInputStream(path.toString());
			// copy it to response's OutputStream
			org.apache.commons.io.IOUtils.copy(is, response.getOutputStream());
			response.setContentType("application/pdf");
			response.flushBuffer();
		} catch (IOException ex) {
			LOGGER.debug("Error writing file to output stream. Filename was '{}'" + fileName + " , " + ex);
			throw new RuntimeException("IOError writing file to output stream");
		} catch (FileSystemException e) {
			e.printStackTrace();
		}

	}

	/**
	 * Helper method for making UserID an int when you got string
	 * 
	 * @param userId
	 * @return
	 */
	private int getUserIdAsInt(String userId) {
		int userIdInt = 0;
		try {
			Id userIdParam = Id.generateId(User.DATA_TYPE, userId);
			StringTokenizer sToken = new StringTokenizer(userIdParam.toExternalString(), "_");
			userIdInt = Integer.parseInt(sToken.nextToken());
		} catch (PersistenceException e1) {
			e1.printStackTrace();
		}
		return userIdInt;
	}

	private int getDataTypeIdAsInt(DataType dataType, String id) {
		int idInt = 0;
		try {
			Id idParam = Id.generateId(dataType, id);
			StringTokenizer sToken = new StringTokenizer(idParam.toExternalString(), "_");
			idInt = Integer.parseInt(sToken.nextToken());
		} catch (PersistenceException e1) {
			e1.printStackTrace();
		}
		return idInt;
	}

	private Id generateId(DataType dataType, String id) {
		Id idAsID = null;
		try {
			idAsID = Id.generateId(dataType, id);
		} catch (PersistenceException e) {
			LOGGER.debug("Failed to generate ID generateId Method: " + e);
			e.printStackTrace();
		}
		return idAsID;
	}

	/**
	 * Finds the first group a student is a member of. Given a list of groupIds
	 * as string array and a users course membership id.
	 * 
	 * @param groupsAsString
	 * @param userMembershipId
	 * @return
	 */
	public Group findGroup(String groupsAsString, Id userMembershipId) {
		Group tempGroup = null;
		List<Group> groups = fromStringToGroups(groupsAsString);
		tempGroup = findGroupWithGivenUserId(groups, userMembershipId);

		return tempGroup;
	}

	private Group findGroupWithGivenUserId(List<Group> groups, Id usersCourseMembershipId) {
		Group dummy = null;
		for (Group group : groups) {
			if (group.isUserAMember(usersCourseMembershipId)) {
				return group;
			}
		}
		return dummy;
	}

	private List<Group> fromStringToGroups(String groupAsString) {
		LOGGER.debug("FromStringToGroups: " + groupAsString);
		List<Group> tempGroups = new ArrayList<>();
		String[] submissionGroups = groupAsString.split("(?<=}),(?=P)");
		if (!submissionGroups[0].equals("")) {
			for (int i = 0; i < submissionGroups.length; i++) {
				LOGGER.debug("Split String: " + submissionGroups[i]);
			}
			LOGGER.debug("Split String: " + submissionGroups[0]);
			String subString = "";
			int index = 0;
			for (int i = 0; i < submissionGroups.length; i++) {
				if(i == submissionGroups.length-1){	
					for (int j = submissionGroups[i].toString().length()-1; j > 0; j--) {
						if(submissionGroups[i].toString().charAt(j) != ','){
							subString = submissionGroups[i].toString().substring(0, j+1);
							index = i;
							LOGGER.debug("INSIDE! i:" + index);
							break;
						}
					}
					
					
				}
				LOGGER.debug("Split String3: " + submissionGroups[i]);
			}
			LOGGER.debug("String on index( "+index+" : " + submissionGroups[index] + " Substring: "+subString);
			submissionGroups[index] = subString;
			LOGGER.debug("Split String X3: " + submissionGroups[submissionGroups.length-1]);
			for (String string : submissionGroups) {
				Group groupX = null;
				try {
					groupX = _groupLoader.loadById(Id.generateId(Group.DATA_TYPE, string));
				} catch (KeyNotFoundException e) {
					e.printStackTrace();
				} catch (PersistenceException e) {
					e.printStackTrace();
				}
				tempGroups.add(groupX);

			}

		}
		return tempGroups;
	}

}
