/**
 * Copyright 2017 Anders Lunde, Robin Sjvoll
 *
 * This code is developed under an experimental paradigm, thus it is important to reuse it with caution. 
 */
package no.ntnu.es.bb.schema;

import java.util.List;

import org.apache.log4j.Logger;

import no.ntnu.es.bb.schema.StudentReview;
import blackboard.persist.Id;
import blackboard.persist.PersistenceException;
import blackboard.persist.KeyNotFoundException;
import blackboard.persist.dao.impl.SimpleDAO;
import blackboard.persist.impl.SimpleSelectQuery;
import blackboard.util.StringUtil;

public class StudentReviewDAO extends SimpleDAO<StudentReview> {
	private static final Logger LOGGER = Logger.getLogger( StudentReviewDAO.class.getName() );
	public StudentReviewDAO() {
		super(StudentReview.class);
	}

	public StudentReviewDAO(Class<StudentReview> cls) {
		super(cls);
	}

	public List<StudentReview> loadAll() {
		return getDAOSupport().loadAll();
	}

	public List<StudentReview> searchById(String id) 
			throws KeyNotFoundException {
		if (!StringUtil.isEmpty(id)) {
			SimpleSelectQuery query = new SimpleSelectQuery(getDAOSupport().getMap());
			//Change column name to whatever the name is in the bean
			query.addWhere("pk1", id);
			return getDAOSupport().loadList(query);
		}
		return null;  
	} 
	
	public List<StudentReview> searchByUserIDAndAssessmentID(Integer userID, Integer assessmentID) 
			throws KeyNotFoundException {
		LOGGER.debug("User ID and Assessment ID "+ userID +" # " + assessmentID);
		if (userID != 0 && assessmentID != 0) {
			SimpleSelectQuery query = new SimpleSelectQuery(getDAOSupport().getMap());
			//Change column name to whatever the name is in the bean
			query.addWhere("peerEvaluatorId", userID);
			query.addWhere("peerAssessmentId", assessmentID);
//			return getDAOSupport().loadList(query).get(0);
			return getDAOSupport().loadList(query);
		}
		return null;  
	} 
	
	
    public StudentReview loadById(long id) {
        StudentReview studentReview = null; 
        try {
        	 Id realId = Id.generateId(StudentReview.DATA_TYPE, id);
        	 studentReview = getDAOSupport().loadById(realId);
		} catch (PersistenceException e1) {
			LOGGER.debug("Failed to load studentReviewID! ID: "+ id +" # " + e1);
			e1.printStackTrace();
		}
       
        LOGGER.debug("StudentReview Loaded succsessfully "+ id +" # " + studentReview);
        if(studentReview!=null){
        	return studentReview;
        }
        else return null;
    } 
    
  //Currently loads the last element in the list (last added)
    public StudentReview loadLast() {
        List<StudentReview> globalSettings;
        globalSettings = getDAOSupport().loadAll();
        if(globalSettings!=null&&!globalSettings.isEmpty()){
            //DEBUGGING
        	for (StudentReview studentReview : globalSettings) {
        		LOGGER.debug("A#"+ studentReview);
			}
        return globalSettings.get(globalSettings.size()-1);
        }
        else return null;
    } 
    
    public void save(StudentReview globalSettings) {
        System.out.println(globalSettings);        
                
        getDAOSupport().persist(globalSettings);
    }
    
    public boolean isInstalled(){
        List<StudentReview> globalSettings = null;
        globalSettings = getDAOSupport().loadAll();
        
   
        if (globalSettings.size() > 0) {
            return true;
        }
        return false;
    }
    
    public void writeDefaults() {
    	StudentReview globalSettings = new StudentReview();
        
        getDAOSupport().persist(globalSettings);
        
    }

	public StudentReview searchByEvaluatorAndUserToEvaluateId(Integer userId, Integer userToEvalId, Integer assessmentId) 
			throws KeyNotFoundException {
		LOGGER.debug("User ID and Assessment ID "+ userId +" # " + userToEvalId);
		if (userId != 0 && userToEvalId != 0) {
			SimpleSelectQuery query = new SimpleSelectQuery(getDAOSupport().getMap());
			//Change column name to whatever the name is in the bean
			query.addWhere("peerEvaluatorId", userId);
			query.addWhere("userToBeEvalId", userToEvalId);
			query.addWhere("peerAssessmentId", assessmentId);
//			return getDAOSupport().loadList(query).get(0);
			return getDAOSupport().load(query);
		}
		return null;  
	} 
}
