/**
 * Copyright 2017 Anders Lunde, Robin Sjvoll
 *
 * This code is developed under an experimental paradigm, thus it is important to reuse it with caution. 
 */
package no.ntnu.es.bb.schema;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardCopyOption;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Enumeration;
import java.util.List;
import java.util.StringTokenizer;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.FileSystemResource;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RequestPart;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.ModelAndView;

import no.ntnu.es.bb.schema.Assessment;
import no.ntnu.es.bb.schema.AssessmentDAO;
import blackboard.base.FormattedText;
import blackboard.data.ExtendedData;
import blackboard.data.ReceiptOptions;
import blackboard.data.content.Content;
import blackboard.data.course.Course;
import blackboard.data.course.CourseMembership;
import blackboard.data.course.Group;
import blackboard.data.user.User;
import blackboard.persist.BbPersistenceManager;
import blackboard.persist.DataType;
import blackboard.persist.Id;
import blackboard.persist.KeyNotFoundException;
import blackboard.persist.PersistenceException;
import blackboard.persist.content.ContentDbLoader;
import blackboard.persist.course.CourseMembershipDbLoader;
import blackboard.persist.course.GroupDbLoader;
import blackboard.platform.context.Context;
import blackboard.platform.context.ContextManager;
import blackboard.platform.context.ContextManagerFactory;
import blackboard.platform.filesystem.FileSystemException;
import blackboard.platform.filesystem.manager.CourseFileManager;
import blackboard.platform.persistence.PersistenceServiceFactory;
import blackboard.platform.servlet.InlineReceiptUtil;

// Denote this class as a Spring Controller
@Controller
public class SubmissionController {
	private static final Logger LOGGER = Logger.getLogger( SubmissionController.class.getName() );
	private final String ASSESSMENT_ID = "assessmentID";
	// Autowire -- a.k.a. auto instantiate -- the DAOs for use in the controller
	@Autowired
	private AssessmentDAO assess_dao;
	
	@Autowired
	private SubmissionDAO submission_dao;
	
	@Autowired
  	private GroupDbLoader _groupLoader;
	

	//Pre submitt to check if submission time is out. Redirecting according to findings.
	@RequestMapping("/submission/preSubmit")
	public void preSubmit(HttpServletRequest request, HttpServletResponse response,
								@RequestParam("content_id") String contentId,
								@RequestParam("course_id") String courseId 
	){
	
		ContentDbLoader loader;
		Id contentID;
		Content courseDoc = null;
		try {
			loader = ContentDbLoader.Default.getInstance();
			contentID = Id.generateId(Content.DATA_TYPE, contentId);
			courseDoc = loader.loadById(contentID);
		} catch (PersistenceException e) {
			LOGGER.debug("Failed to load Content in /submit ");
			e.printStackTrace();
		}

		ExtendedData courseEx = courseDoc.getExtendedData();
		String assessment_id = courseEx.getValue(ASSESSMENT_ID);
		int assessment_id_int = getDataTypeIdAsInt(Assessment.DATA_TYPE, assessment_id);
		LOGGER.debug("CourseDoc: " + courseDoc + " ExtendedData: " + courseEx + " assessmentID: " + assessment_id);
		Assessment assessment = assess_dao.loadById(Integer.parseInt(assessment_id));

		//Finds the correct submission id, if no submission is found the id is set to -1
		int submission_id = -1;
		//Getting userID
		ContextManager contextManager = ContextManagerFactory.getInstance();
		Context ctx = contextManager.getContext();
		User user = ctx.getUser();
		int userIdInt = Integer.parseInt(new StringTokenizer(user.getId().toExternalString(), "_").nextToken());
		
		int groupSubmissionId = -1;
		boolean isGroupSubmission = assessment.getAssignmentType().equals("G");
		
		if(isGroupSubmission){
			
			CourseMembershipDbLoader membershipLoader = null;
			CourseMembership membership = null;
			Id courseIdParam = generateId(Course.DATA_TYPE, courseId);
			
			try {
				membershipLoader = CourseMembershipDbLoader.Default.getInstance();
				membership = membershipLoader.loadByCourseAndUserId(courseIdParam, user.getId());
			} catch (KeyNotFoundException e1) {
				LOGGER.debug("Failed to get membership or loader: " + e1);
				e1.printStackTrace();
			} catch (PersistenceException e1) {
				LOGGER.debug("Failed to get membership or loader: " + e1);
				e1.printStackTrace();
			}
			
			//Finds the first group that the student is a member of
			Group group = findGroup(assessment.getSubmissionGroups(), membership.getId());
			
			if(group != null){
				groupSubmissionId = getDataTypeIdAsInt(Group.DATA_TYPE, group.getId().toExternalString());
				
			}
		}
			
		//The submission
		Submission submission = null;
		
		//See if group has submitted a submission
		if(isGroupSubmission){
			try {
				submission = submission_dao.searchByGroupIDAndAssessmentID(groupSubmissionId,assessment_id_int);
			} catch (KeyNotFoundException e) {
				LOGGER.debug("Submission not found");
				e.printStackTrace();
			}
		}
	
		//If there are no group assessments
		if (submission == null) {
			try {
				submission = submission_dao.searchByUserIDAndAssessmentID(userIdInt, assessment_id_int);
			} catch (KeyNotFoundException e) {
				LOGGER.debug("Submission not found");
				e.printStackTrace();
			}
		}
		
		//If the submission exsists the correct ID is set
		if(submission != null){
			submission_id = submission.getPropertyId();
			LOGGER.debug("Submission exsist: " + submission_id);
		}
		
		// Check if submission time has ended, if ended redirects to review_of_review jsp
		// SimpleDateFormats: YYYY-MM-DD HH:MM:SS
		String currentTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS").format(Calendar.getInstance().getTime());
		// Date assessmentEndTime = assessment.getSubmissionDateEnd().getTime();
		String assessmentEndTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS")
				.format(assessment.getSubmissionDateEnd().getTime());
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");

		LOGGER.debug("CurrentTime:" + currentTime + " AssessmentEndTime: " + assessmentEndTime);

		try {
			if (!sdf.parse(currentTime).before(sdf.parse(assessmentEndTime))) {
				ReceiptOptions ro = new ReceiptOptions();
				ro.addWarningMessage("Submission Time has ended. Displaying Evaluation");

				InlineReceiptUtil.addReceiptToRequest(request, ro);
				
				response.sendRedirect("../review/reviewOfReview?course_id=" + courseId + "&content_id=" + contentId + "&assessment_id=" + assessment.getPropertyId() + "&submission_id=" + submission_id);
			}else{
				response.sendRedirect("submit?course_id=" + courseId + "&content_id=" + contentId);

			}
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ParseException e) {
			e.printStackTrace();
		}
	}
	
	//Method ran when user submits his/hers submission
	@RequestMapping("/submission/submit")
	public ModelAndView submit(HttpServletRequest request, HttpServletResponse response,
								@RequestParam("content_id") String contentId,
								@RequestParam("course_id") String courseId 
	){
		
		// In order to get the asssessment ID (which is not sent through the
		// URI) we get it from Content extra data.
		ContentDbLoader loader;
		Id contentID;
		Content courseDoc = null;
		try {
			loader = ContentDbLoader.Default.getInstance();
			contentID = Id.generateId(Content.DATA_TYPE, contentId);
			courseDoc = loader.loadById(contentID);
		} catch (PersistenceException e) {
			LOGGER.debug("Failed to load Content in /submit ");
			e.printStackTrace();
		}

		ExtendedData courseEx = courseDoc.getExtendedData();
		String assessment_id = courseEx.getValue(ASSESSMENT_ID);
		int assessment_id_int = getDataTypeIdAsInt(Assessment.DATA_TYPE, assessment_id);
		LOGGER.debug("CourseDoc: " + courseDoc + " ExtendedData: " + courseEx + " assessmentID: " + assessment_id);
		Assessment assessment = assess_dao.loadById(Integer.parseInt(assessment_id));

		// Create a new ModelAndView Object. The constructor takes a string that corresponds to the name of the jsp file the object should be routed to
		ModelAndView mv = new ModelAndView("/submission/submit");
		
		//Getting userID
		ContextManager contextManager = ContextManagerFactory.getInstance();
		Context ctx = contextManager.getContext();
		User user = ctx.getUser();
		int userIdInt = Integer.parseInt(new StringTokenizer(user.getId().toExternalString(), "_").nextToken());
		 
		//Original assessment code location

		// get the string name from the database, convert it to FormattedText
		// and store it in the name variable.
		String assessment_name = assessment.getName();

		// get the string name from the database, convert it to FormattedText and store it in the name variable.
//		FormattedText assessment_instructions = FormattedText.toFormattedText(assessment.getInstructions());
		String assessment_instructions = assessment.getInstructions();
		
		//Group submission
		//Check if the assessment is a group assessment. If so, the submission will have the group ID as a stored variable and all members of that group will see the same submision.
		mv.addObject("isGroupAssessment", false);
		mv.addObject("groupName", "No Groups found");
		
		int groupSubmissionId = -1;
		boolean isGroupSubmission = assessment.getAssignmentType().equals("G");
		mv.addObject("isGroupSubmission", isGroupSubmission);
		
		if(isGroupSubmission){
			mv.addObject("isGroupAssessment", true);
			
			CourseMembershipDbLoader membershipLoader = null;
			CourseMembership membership = null;
			Id courseIdParam = generateId(Course.DATA_TYPE, courseId);
			
			try {
				membershipLoader = CourseMembershipDbLoader.Default.getInstance();
				membership = membershipLoader.loadByCourseAndUserId(courseIdParam, user.getId());
			} catch (KeyNotFoundException e1) {
				LOGGER.debug("Failed to get membership or loader: " + e1);
				e1.printStackTrace();
			} catch (PersistenceException e1) {
				LOGGER.debug("Failed to get membership or loader: " + e1);
				e1.printStackTrace();
			}
			
			//Finds the first group that the student is a member of
			Group group = findGroup(assessment.getSubmissionGroups(), membership.getId());
			
			if(group != null){
				mv.addObject("groupName", group.getTitle());
				groupSubmissionId = getDataTypeIdAsInt(Group.DATA_TYPE, group.getId().toExternalString());
				
			}
		}
		mv.addObject("groupSubId", groupSubmissionId);
		mv.addObject("submission_files", null);
		

		//Dummy value when no submission is saved before.
		Integer emptySubID = null;
		
		// ModelAndView.addObject adds the objects you wish to pass to the jsp
		mv.addObject("assessment_name", assessment_name);
		mv.addObject("assessment_id", assessment_id);
		mv.addObject("assessment_instructions", assessment_instructions);
		mv.addObject("submission_text", "");
		mv.addObject("comment_text", "");
		mv.addObject("edit", false);
		mv.addObject("submission_id", emptySubID);
		mv.addObject("content_id", contentId);
		
		//Sumbission info, if one exsists already load it
		Submission submission = null;
		
		//See if group has submitted a submision
		if(isGroupSubmission){
			try {
				submission = submission_dao.searchByGroupIDAndAssessmentID(groupSubmissionId,assessment_id_int);
				if (submission != null) {
					mv.addObject("submission_text", FormattedText.toFormattedText(submission.getSubmissionText()));
					mv.addObject("comment_text", FormattedText.toFormattedText(submission.getCommentText()));
					mv.addObject("edit", true);
					mv.addObject("submission_id", submission.getPropertyId());
				}
			} catch (KeyNotFoundException e) {
				LOGGER.debug("Submission not found");
				e.printStackTrace();
			}
		}
	
		//If there are no group assessments
		if (submission == null) {
			try {
				submission = submission_dao.searchByUserIDAndAssessmentID(userIdInt, assessment_id_int);
				mv.addObject("submission_text", FormattedText.toFormattedText(submission.getSubmissionText()));
				mv.addObject("comment_text", FormattedText.toFormattedText(submission.getCommentText()));
				mv.addObject("edit", true);
				mv.addObject("submission_id", submission.getPropertyId());
			} catch (KeyNotFoundException e) {
				LOGGER.debug("Submission not found");
				e.printStackTrace();
			}
		}
		
		//Loading file
		BbPersistenceManager bbPm = PersistenceServiceFactory.getInstance().getDbPersistenceManager();
		
		//Loading file Success attempt (uses regular file structure):
		CourseFileManager crsFileMng = new CourseFileManager();
		
		mv.addObject("filePath", "_no path_");
		mv.addObject("currentFileName", "No File");
		if (submission != null) {
			mv.addObject("currentFileName", "No File [0]");
			try {
				File root = crsFileMng.getRootDirectory(Integer.toString(getDataTypeIdAsInt(Course.DATA_TYPE,courseId)));
				
				Path path = Paths.get(
						root.getPath() + "/b2_peer_assessment/assessment_id_"+Integer.toString(getDataTypeIdAsInt(Assessment.DATA_TYPE,assessment_id))+"/submissions_files/pk1_" + submission.getPropertyId());
				mv.addObject("currentFileName", "No File [1]");
				mv.addObject("filePath", path);
				
				File submittedfile = new File(path.toString());
				    
				
				if (Files.exists(path)) {
					mv.addObject("currentFileName", "File exsist [2]");
					mv.addObject("currentFileName", submittedfile.getName());
				}
			} catch (FileSystemException e) {
				LOGGER.debug("FAILED TO GET ROOT WHEN LOADING SUBMISSION: " + e);
				e.printStackTrace();
			}
		}
		
		//Adding assessment_files
		mv.addObject("assessment_files", null);
		
		//Find stored assessment files
		try {
			File root = crsFileMng.getRootDirectory(Integer.toString(getDataTypeIdAsInt(Course.DATA_TYPE, courseId)));
			Path assessmentPath = Paths.get(root.getPath() + "/b2_peer_assessment/assessment_id_"
					+ Integer.toString(getDataTypeIdAsInt(Assessment.DATA_TYPE, assessment_id)) + "/assessment_files");

			List<File> submittedfiles = new ArrayList<>();
			File folder = new File(assessmentPath.toString());
			for (File assessmentFile : folder.listFiles()) {
				LOGGER.debug("File in storage " + assessmentFile);
				submittedfiles.add(assessmentFile);
			}

			if (submittedfiles.size() > 0) {
				mv.addObject("assessment_files", submittedfiles);
			}
			//
		} catch (FileSystemException e) {
			LOGGER.debug("FAILED TO GET ROOT WHEN LOADING SUBMISSION Assessment files: " + e);
			e.printStackTrace();
		}

		LOGGER.debug("Assessment is now loaded in submit view");
		
		// Returning the ModelAndView tells spring to find the JSP and set these objects in the context for the request.
		return mv;   
	}

		@RequestMapping("/submission/saveSubmission")
		private void saveSubmission( HttpServletRequest request, HttpServletResponse response, 
				@RequestParam("submissiontext") String submission_text,
				@RequestParam("submission_comments_text") String submission_comments_text,
				@RequestParam("user_id") String userId,
				@RequestParam("assessment_id") int assessment_id, 
				@RequestParam("course_id") String courseId, 
				@RequestParam("content_id") String contentId, 
				@RequestParam("edit") boolean edit, 
				@RequestParam("submission_id") Integer submission_id,
				@RequestParam("group_submission_id") Integer group_submission_id,
				@RequestPart(value="submission_file_LocalFile0", required=false) MultipartFile multipartFile
		) {
		
		Id content_ID = generateId(Content.DATA_TYPE, contentId);
		Id user_ID = generateId(User.DATA_TYPE, userId);
		Id course_ID = generateId(Course.DATA_TYPE, courseId);
		// dao is already instantiated, so load the current instance, since properties only has one record that we maintain. If using a multiple record database, you would
		// want to create a new Properties object and set all the parameters associated with that object.
		Submission submission;
		if(edit){
			submission = submission_dao.loadById(submission_id);
		}else{
			submission = new Submission();
		}
		
        //set all prefs to incoming request parameter values
		submission.setSubmissionText(submission_text);
		submission.setCommentText(submission_comments_text);        	
   
		int userIdInt = 0;
		
        try {
			Id userIdParam = Id.generateId(User.DATA_TYPE, userId);
			StringTokenizer sToken = new StringTokenizer(userIdParam.toExternalString(), "_");
			
			userIdInt = Integer.parseInt(sToken.nextToken());
		} catch (PersistenceException e1) {
			e1.printStackTrace();
		}
        
        submission.setSubmissionUserId(userIdInt);
        
        submission.setPeerAssessmentId(assessment_id);
        
        submission.setGroupSubmissionId(group_submission_id);
        
        
        //save the submission
        submission_dao.save(submission);

		LOGGER.debug("Going to try to save the File:"+ multipartFile.getName());

		/**
		 * This whole try method uses the inner file structure of a course to store the submitted files.
		 * This might not be a good idea for a b2 in prodution but it works well for testing purposes.
		 * There is also a permission in the manifest connected to this "try" code, needed for writing/reading the file.
		 */
		CourseFileManager crsFileMng = new CourseFileManager();
		int newSubmissionID = submission_dao.loadLast().getPropertyId();
		
		try {
			File file = convert(multipartFile);
			LOGGER.error("File found for LocalFile0: " + file);
			
			LOGGER.error("Course ID: " + Integer.toString(getDataTypeIdAsInt(Course.DATA_TYPE,courseId)));
			File root = crsFileMng.getRootDirectory(Integer.toString(getDataTypeIdAsInt(Course.DATA_TYPE,courseId)));

			//If the default peer review folders does not exsist
			Path path = Paths.get(root.getPath()+"/b2_peer_assessment/assessment_id_"+Integer.toString(getDataTypeIdAsInt(Assessment.DATA_TYPE,Integer.toString(assessment_id)))+"/submissions_files");
			
			try {
				Path fileToMovePath = 
					      Paths.get(file.getAbsolutePath());
				Path targetPath = Paths.get(path+"/"+"pk1_"+ newSubmissionID);
				Files.move(fileToMovePath, targetPath,StandardCopyOption.REPLACE_EXISTING);
			} catch (IOException e) {
				LOGGER.debug("FAILED TO MOVE FILE: "+ e);
				e.printStackTrace();
			}
			
			
			// Debugging output to display all files inside our submission_files
			// folder:
			// File folder = new File(path.toString()+"/");
			File folder = new File(root.getPath() + "/b2_peer_assessment/assessment_id_"
					+ Integer.toString(getDataTypeIdAsInt(Assessment.DATA_TYPE, Integer.toString(assessment_id)))
					+ "/submissions_files/");
			File[] listOfFiles = folder.listFiles();
			LOGGER.debug("Running List of files on " + path + "/" + " RESULT: " + listOfFiles + " With Length: "
					+ listOfFiles.length);
			for (int i = 0; i < listOfFiles.length; i++) {
				if (listOfFiles[i].isFile()) {
					LOGGER.debug("File " + listOfFiles[i].getName());
				} else if (listOfFiles[i].isDirectory()) {
					LOGGER.debug("Directory " + listOfFiles[i].getName());
				}
			}
		} catch (FileSystemException e2) {
			e2.printStackTrace();
		} catch (IOException e1) {
			LOGGER.error("FAILED TO CONVERT MULTIPART FILE");
			e1.printStackTrace();
		} catch (NullPointerException e1) {
			LOGGER.error("NO FILE FOUND");
			e1.printStackTrace();
		} 
		
        try {
        	ReceiptOptions ro = new ReceiptOptions();
        	if(edit){
        		ro.addSuccessMessage("Submission Successfully Updated.");
        	}else{
        		ro.addSuccessMessage("Submission Successfully Submitted.");
        	}
        	
        	InlineReceiptUtil.addReceiptToRequest(request, ro); 
        	LOGGER.debug("props are now saved_3!");
        	response.sendRedirect("submit?course_id="+courseId+"&content_id="+contentId);
		} catch (IOException e) {
			e.printStackTrace();
		}
    }
		
	@RequestMapping(value = "/submission/fetchFile", method = RequestMethod.GET)
	@ResponseBody
	public FileSystemResource getFile(
			@RequestParam("file_name") String fileName,
			@RequestParam("assessment_id") String assessmentId,
			@RequestParam("submission_id") String submissionId,
			@RequestParam("course_id") String courseId
			) {
		CourseFileManager crsFileMng = new CourseFileManager();
		File root;
		File submittedfile = null;
		try {
			root = crsFileMng.getRootDirectory(Integer.toString(getDataTypeIdAsInt(Course.DATA_TYPE,courseId)));
			Path path = Paths.get(
					root.getPath() + "/b2_peer_assessment/assessment_id_"+Integer.toString(getDataTypeIdAsInt(Assessment.DATA_TYPE,assessmentId))+"/submissions_files/pk1_" + submissionId);
			submittedfile = new File(path.toString());
		} catch (FileSystemException e) {
			LOGGER.error("FAILED TO DOWNLOADE FILE");
			e.printStackTrace();
		}
		
		return new FileSystemResource(submittedfile); 
	}
	
	//For downloading file to do assignment
	@RequestMapping(value = "/submission/downloadFile")
	public void getFile(
			@RequestParam("index") int index,
			@RequestParam("assessment_id") String assessmentId,
			@RequestParam("course_id") String courseId,
	    HttpServletResponse response) {
		CourseFileManager crsFileMng = new CourseFileManager();
		List<File> submittedfiles = new ArrayList<>();
		File root;
	    try {
	      // get your file as InputStream
	    	root = crsFileMng.getRootDirectory(Integer.toString(getDataTypeIdAsInt(Course.DATA_TYPE,courseId)));
			Path assessmentPath = Paths.get(root.getPath() + "/b2_peer_assessment/assessment_id_"
					+ Integer.toString(getDataTypeIdAsInt(Assessment.DATA_TYPE, assessmentId)) + "/assessment_files");

			File folder = new File(assessmentPath.toString());
			for (File assessmentFile : folder.listFiles()) {
				LOGGER.debug("File in storage " + assessmentFile);
				submittedfiles.add(assessmentFile);
			}
	      InputStream is = new FileInputStream(submittedfiles.get(index).getAbsolutePath());
	      // copy it to response's OutputStream
	      org.apache.commons.io.IOUtils.copy(is, response.getOutputStream());
	      response.setContentType("application/pdf");
	      response.flushBuffer();
	    } catch (IOException ex) {
	      LOGGER.debug("Error writing file to output stream. Filename was '{}'" + submittedfiles.get(index) + " , "+ ex);
	      throw new RuntimeException("IOError writing file to output stream");
	    } catch (FileSystemException e) {
			e.printStackTrace();
		}

	}
	
	private int getDataTypeIdAsInt(DataType dataType, String id){
		int idInt = 0;
		try {
			Id idParam = Id.generateId(dataType, id);
			StringTokenizer sToken = new StringTokenizer(idParam.toExternalString(), "_");
			idInt = Integer.parseInt(sToken.nextToken());
		} catch (PersistenceException e1) {
			e1.printStackTrace();
		}
		return idInt;
	}
	
	private Id generateId(DataType dataType , String id ){
		Id idAsID = null;
		try {
			idAsID = Id.generateId(dataType, id);
		} catch (PersistenceException e) {
			LOGGER.debug("Failed to generate ID generateId Method: " + e);
			e.printStackTrace();
		}
		return idAsID;
	}
	
	public File convert(MultipartFile file) throws IOException
	{    
	    File convFile = new File(file.getOriginalFilename());
	    convFile.createNewFile(); 
	    FileOutputStream fos = new FileOutputStream(convFile); 
	    fos.write(file.getBytes());
	    fos.close(); 
	    return convFile;
	}
	
	/**
	 * Finds the first group a student is a member of. Given a list of groupIds as string array and a users course membership id.
	 * @param groupsAsString
	 * @param userMembershipId
	 * @return
	 */
	public Group findGroup(String groupsAsString, Id userMembershipId){
		Group tempGroup = null;
		List<Group> groups = fromStringToGroups(groupsAsString);
		tempGroup = findGroupWithGivenUserId(groups, userMembershipId);
		
		return tempGroup;
	}
	
	private Group findGroupWithGivenUserId(List<Group> groups, Id usersCourseMembershipId) {
		Group dummy = null;
		for (Group group : groups) {
			if(group.isUserAMember(usersCourseMembershipId)){
				return group;
			}
		}
		return dummy;
	}

	private List<Group> fromStringToGroups(String groupAsString){
		LOGGER.debug("FromStringToGroups: " + groupAsString);
		List<Group> tempGroups = new ArrayList<>();
		String[] submissionGroups = groupAsString.split("(?<=}),(?=P)");
		if (!submissionGroups[0].equals("")) {
			for (int i = 0; i < submissionGroups.length; i++) {
				LOGGER.debug("Split String: " + submissionGroups[i]);
			}
			LOGGER.debug("Split String: " + submissionGroups[0]);
			
			String subString = "";
			int index = 0;
			for (int i = 0; i < submissionGroups.length; i++) {
				if(i == submissionGroups.length-1){	
					for (int j = submissionGroups[i].toString().length()-1; j > 0; j--) {
						if(submissionGroups[i].toString().charAt(j) != ','){
							subString = submissionGroups[i].toString().substring(0, j+1);
							index = i;
							LOGGER.debug("INSIDE! i:" + index);
							break;
						}
					}
					
					
				}
				LOGGER.debug("Split String4: " + submissionGroups[i]);
			}
			LOGGER.debug("String1 on index( "+index+" : " + submissionGroups[index] + " Substring: "+subString);
			submissionGroups[index] = subString;
			LOGGER.debug("Split String X4: " + submissionGroups[submissionGroups.length-1]);
			for (String string : submissionGroups) {
				Group groupX = null;
				try {
					groupX = _groupLoader.loadById(Id.generateId(Group.DATA_TYPE, string));
				} catch (KeyNotFoundException e) {
					e.printStackTrace();
				} catch (PersistenceException e) {
					e.printStackTrace();
				}
				tempGroups.add(groupX);
				
			}
			
		}
		return tempGroups;
	}
}

