/**
 * Copyright 2017 Anders Lunde, Robin Sjvoll
 *
 * This code is developed under an experimental paradigm, thus it is important to reuse it with caution. 
 */
package no.ntnu.es.bb.schema;

import java.util.List;

import org.apache.log4j.Logger;

import no.ntnu.es.bb.schema.Submission;
import blackboard.persist.Id;
import blackboard.persist.PersistenceException;
import blackboard.persist.KeyNotFoundException;
import blackboard.persist.dao.impl.SimpleDAO;
import blackboard.persist.impl.SimpleSelectQuery;
import blackboard.util.StringUtil;

public class SubmissionDAO extends SimpleDAO<Submission> {
	private static final Logger LOGGER = Logger.getLogger( SubmissionDAO.class.getName() );
	public SubmissionDAO() {
		super(Submission.class);
	}

	public SubmissionDAO(Class<Submission> cls) {
		super(cls);
	}

	public List<Submission> loadAll() {
		return getDAOSupport().loadAll();
	}

	public List<Submission> searchById(String id) 
			throws KeyNotFoundException {
		if (!StringUtil.isEmpty(id)) {
			SimpleSelectQuery query = new SimpleSelectQuery(getDAOSupport().getMap());
			//Change column name to whatever the name is in the bean
			query.addWhere("pk1", id);
			return getDAOSupport().loadList(query);
		}
		return null;  
	} 
	
	public Submission searchByUserIDAndAssessmentID(Integer userID, Integer assessmentID) 
			throws KeyNotFoundException {
		LOGGER.debug("User ID and Assessment ID "+ userID +" # " + assessmentID);
		if (userID != 0 && assessmentID != 0) {
			SimpleSelectQuery query = new SimpleSelectQuery(getDAOSupport().getMap());
			//Change column name to whatever the name is in the bean
			query.addWhere("submissionUserId", userID);
			query.addWhere("peerAssessmentId", assessmentID);
//			return getDAOSupport().loadList(query).get(0);
			return getDAOSupport().load(query);
		}
		return null;  
	} 
	
	public Submission searchByGroupIDAndAssessmentID(Integer groupID, Integer assessmentID) 
			throws KeyNotFoundException {
		LOGGER.debug("GroupID and Assessment ID "+ groupID +" # " + assessmentID);
		if (groupID > 0 && assessmentID != 0) {
			SimpleSelectQuery query = new SimpleSelectQuery(getDAOSupport().getMap());
			//Change column name to whatever the name is in the bean
			query.addWhere("groupSubmissionId", groupID);
			query.addWhere("peerAssessmentId", assessmentID);
//			return getDAOSupport().loadList(query).get(0);
			return getDAOSupport().load(query);
		}
		return null;  
	} 
	
	public List<Submission> findAllWithAssessmentID(Integer assessmentID) 
			throws KeyNotFoundException {
		LOGGER.debug("Find all Assessments with ID: " + assessmentID);
		if (assessmentID != 0) {
			SimpleSelectQuery query = new SimpleSelectQuery(getDAOSupport().getMap());
			query.addWhere("peerAssessmentId", assessmentID);
			return getDAOSupport().loadList(query);
		}
		return null;  
	} 
	
	
    public Submission loadById(long id) {
        Submission submission = null; 
        try {
        	 Id realId = Id.generateId(Submission.DATA_TYPE, id);
        	 submission = getDAOSupport().loadById(realId);
		} catch (PersistenceException e1) {
			LOGGER.debug("Failed to load courseID! ID: "+ id +" # " + e1);
			e1.printStackTrace();
		}
       
        LOGGER.debug("Submission Loaded succsessfully "+ id +" # " + submission);
        if(submission!=null){
        	return submission;
        }
        else return null;
    } 
    
  //Currently loads the last element in the list (last added)
    public Submission loadLast() {
        List<Submission> globalSettings;
        globalSettings = getDAOSupport().loadAll();
        if(globalSettings!=null&&!globalSettings.isEmpty()){
            //DEBUGGING
        	for (Submission submission : globalSettings) {
        		LOGGER.debug("A#"+ submission);
			}
        return globalSettings.get(globalSettings.size()-1);
        }
        else return null;
    } 
    
    public void save(Submission globalSettings) {
        System.out.println(globalSettings);        
                
        getDAOSupport().persist(globalSettings);
    }
    
    public boolean isInstalled(){
        List<Submission> globalSettings = null;
        globalSettings = getDAOSupport().loadAll();
        
   
        if (globalSettings.size() > 0) {
            return true;
        }
        return false;
    }
    
    public void writeDefaults() {
    	Submission globalSettings = new Submission();
        
        getDAOSupport().persist(globalSettings);
        
    }
}
