/**
 * Copyright 2017 Anders Lunde, Robin Sjvoll
 *
 * This code is developed under an experimental paradigm, thus it is important to reuse it with caution. 
 */
package no.ntnu.es.bb.schema;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.util.List;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.ModelAndView;

import no.ntnu.es.bb.schema.Assessment;
import no.ntnu.es.bb.schema.AssessmentDAO;
import blackboard.db.BbDatabase;
import blackboard.db.ConnectionManager;
import blackboard.db.ConnectionNotAvailableException;
import blackboard.platform.spring.beans.annotations.UserAuthorization;

// Denote this class as a Spring Controller
@Controller
public class ToolsController {
	private static final Logger LOGGER = Logger.getLogger( ToolsController.class.getName() );
	// Autowire -- a.k.a. auto instantiate -- the asessmentDAO object for use in the controller
	@Autowired
	private AssessmentDAO _dao;
	
	// This is a system too, so ensure the user running the code has permission to view the System Admin panel
	@UserAuthorization("system.admin.VIEW")
	
	//Ran when the instructor goes to the peer assessment tab in course tools, fetches all assessments.
	@RequestMapping("/tools/show_all_reviews")
	public ModelAndView showAllReviews( HttpServletRequest request, HttpServletResponse response,
			@RequestParam(value="select", required=false ) String select,
			@RequestParam(value="from", required=false) String from
	){
		// Create a new ModelAndView Object. The constructor takes a string that corresponds to the name of the jsp file the object should be routed to
		ModelAndView mv = new ModelAndView("tools/show_all_reviews");
		
		// Since we autowired the DAO, we can simply call the load method.
		List<Assessment> assessments = _dao.loadAll();
		
		
//		/**
//		 * TEST AREA
//		 */
//		String resultFromQuery = "";
//		
//		ConnectionManager cManager = null;
//        Connection conn = null;
//        StringBuffer queryString = new StringBuffer("");
//        PreparedStatement selectQuery = null;
//		
//        try {
//		
//		cManager = BbDatabase.getDefaultInstance().getConnectionManager();
//        conn = cManager.getConnection();
////
////        queryString.append("select count(distinct user_pk1) from activity_accumulator ");
////        queryString.append("where data = 'Login succeeded.' ");
////        queryString.append("and timestamp between ? and ? ");
//        
////      THE MAGIC QUERY: :P 
//        if(select != null && from != null){
//        	queryString.append("select " + select + "from " + from);  
//        }else{
//        	queryString.append("select * from users");        	
//        }
//        
//        selectQuery = conn.prepareStatement(queryString.toString(), ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
////        selectQuery.setDate(1, startDateSql);
////        selectQuery.setDate(2, endDateSql);
//        
//        ResultSet rSet = selectQuery.executeQuery();
//
//        ResultSetMetaData rsmd = rSet.getMetaData();
//        int columnsNumber = rsmd.getColumnCount();
//        
//        while(rSet.next()){
//        	
//        	for (int i = 1; i <= columnsNumber; i++) {
//                if (i > 1){
//                	resultFromQuery += ",  ";
//                	resultFromQuery += "<br />"; //Newline in html
//                }
//                String columnValue = rSet.getString(i);
//                resultFromQuery += rsmd.getColumnName(i) +": " +columnValue;
//     
//            }
//        	resultFromQuery += "<br /><br />"; //Newline in html
////        	resultFromQuery += "\n";
////        	totalLogins = rSet.getInt(1);
//         
//        }
//        
//        rSet.close();
//        
//        selectQuery.close();
//
//    } catch (java.sql.SQLException sE){
//    	
//    	LOGGER.error( sE.getMessage() ) ;
//    	
//    	throw new IllegalStateException("Unable to query database for total logins.") ;
//        
//    } catch (ConnectionNotAvailableException cE){
//    	
//    	LOGGER.error( cE.getMessage() ) ;
//    	
//    	throw new IllegalStateException("Unable to query database for total logins.") ;
//        
//    } finally {
//    	
//        if(conn != null){
//            cManager.releaseConnection(conn);
//        }
//        
//    }
//		
//
//		/**
//		 * TEST AREA END
//		 */
        
		// ModelAndView.addObject adds the objects you wish to pass to the jsp
		mv.addObject("assessments", assessments);
		mv.addObject("resultFromQuery", "");
//		mv.addObject("resultFromQuery", resultFromQuery);

		LOGGER.debug("assessment list is now loaded!");
		
		// Returning the ModelAndView tells spring to find the JSP and set these objects in the context for the request.
		return mv;   
	}

	@RequestMapping("/tools/show_all_reviews_module")
	public ModelAndView module( HttpServletRequest request, HttpServletResponse response,
			@RequestParam(value="select", required=false ) String select,
			@RequestParam(value="from", required=false) String from
	){
		// Create a new ModelAndView Object. The constructor takes a string that corresponds to the name of the jsp file the object should be routed to
		ModelAndView mv = new ModelAndView("tools/show_all_reviews_module");
		
		// Since we autowired the DAO, we can simply call the load method.
		List<Assessment> assessments = _dao.loadAll();
		
		// ModelAndView.addObject adds the objects you wish to pass to the jsp
		mv.addObject("assessments", assessments);

		LOGGER.debug("assessment module list is now loaded!");
		
		// Returning the ModelAndView tells spring to find the JSP and set these objects in the context for the request.
		return mv;   
	}
	

}
