/**
 * Copyright 2017 Anders Lunde, Robin Sjvoll
 *
 * This code is developed under an experimental paradigm, thus it is important to reuse it with caution. 
 */
package no.ntnu.es.bb.schema.rubric;

import java.util.List;

import org.apache.log4j.Logger;

import blackboard.persist.Id;
import blackboard.persist.PersistenceException;
import blackboard.persist.KeyNotFoundException;
import blackboard.persist.dao.impl.SimpleDAO;
import blackboard.persist.impl.SimpleSelectQuery;
import blackboard.util.StringUtil;

public class RubricCellDAO extends SimpleDAO<RubricCell> {
	private static final Logger LOGGER = Logger.getLogger( RubricCellDAO.class.getName() );
	public RubricCellDAO() {
		super(RubricCell.class);
	}

	public RubricCellDAO(Class<RubricCell> cls) {
		super(cls);
	}

	public List<RubricCell> loadAll() {
		return getDAOSupport().loadAll();
	}

	public List<RubricCell> searchById(String id) 
			throws KeyNotFoundException {
		if (!StringUtil.isEmpty(id)) {
			SimpleSelectQuery query = new SimpleSelectQuery(getDAOSupport().getMap());
			//Change column name to whatever the name is in the bean
			query.addWhere("pk1", id);
			return getDAOSupport().loadList(query);
		}
		return null;  
	}
	
	public RubricCell searchByColAndRowId(int rubricColumnID, int rubricRowID) {
		SimpleSelectQuery query = new SimpleSelectQuery(getDAOSupport().getMap());
		//Change column name to whatever the name is in the bean
		query.addWhere("rubricColumnId", rubricColumnID);
		query.addWhere("rubricRowId", rubricRowID);
		List<RubricCell> cells = getDAOSupport().loadList(query);
		if(cells.size() == 0){
			return null;
		}
		return cells.get(0);
	}
	
	
    public RubricCell loadById(long id) {
        RubricCell rubric = null; 
        try {
        	 Id realId = Id.generateId(RubricCell.DATA_TYPE, id);
        	 rubric = getDAOSupport().loadById(realId);
		} catch (PersistenceException e1) {
			LOGGER.debug("Failed to load courseID! ID: "+ id +" # " + e1);
			e1.printStackTrace();
		}
       
        LOGGER.debug("RubricCell Loaded succsessfully "+ id +" # " + rubric);
        if(rubric!=null){
        	return rubric;
        }
        else return null;
    } 
    
  //Currently loads the last element in the list (last added)
    public RubricCell loadLast() {
        List<RubricCell> globalSettings;
        globalSettings = getDAOSupport().loadAll();
        if(globalSettings!=null&&!globalSettings.isEmpty()){
            //DEBUGGING
        	for (RubricCell rubric : globalSettings) {
        		LOGGER.debug("A#"+ rubric);
			}
        return globalSettings.get(globalSettings.size()-1);
        }
        else return null;
    } 
    
    public void save(RubricCell globalSettings) {
        System.out.println(globalSettings);        
                
        getDAOSupport().persist(globalSettings);
    }
    
    public boolean isInstalled(){
        List<RubricCell> globalSettings = null;
        globalSettings = getDAOSupport().loadAll();
        
   
        if (globalSettings.size() > 0) {
            return true;
        }
        return false;
    }
    
    public void writeDefaults() {
    	RubricCell globalSettings = new RubricCell();
        
        getDAOSupport().persist(globalSettings);
        
    }

	
}
