/**
 * Copyright 2017 Anders Lunde, Robin Sjvoll
 *
 * This code is developed under an experimental paradigm, thus it is important to reuse it with caution. 
 */
package no.ntnu.es.bb.schema.rubric;

import java.util.List;

import org.apache.log4j.Logger;

import blackboard.persist.Id;
import blackboard.persist.PersistenceException;
import blackboard.persist.KeyNotFoundException;
import blackboard.persist.dao.impl.SimpleDAO;
import blackboard.persist.impl.SimpleSelectQuery;
import blackboard.util.StringUtil;

public class RubricColumnDAO extends SimpleDAO<RubricColumn> {
	private static final Logger LOGGER = Logger.getLogger( RubricColumnDAO.class.getName() );
	public RubricColumnDAO() {
		super(RubricColumn.class);
	}

	public RubricColumnDAO(Class<RubricColumn> cls) {
		super(cls);
	}

	public List<RubricColumn> loadAll() {
		return getDAOSupport().loadAll();
	}
	
	public List<RubricColumn> searchByRubricId(Integer id) 
			throws KeyNotFoundException {
		if (id != null) {
			SimpleSelectQuery query = new SimpleSelectQuery(getDAOSupport().getMap());
			//Change column name to whatever the name is in the bean
			query.addWhere("rubricId", id);
			return getDAOSupport().loadList(query);
		}
		return null;  
	} 

	public List<RubricColumn> searchById(String id) 
			throws KeyNotFoundException {
		if (!StringUtil.isEmpty(id)) {
			SimpleSelectQuery query = new SimpleSelectQuery(getDAOSupport().getMap());
			//Change column name to whatever the name is in the bean
			query.addWhere("pk1", id);
			return getDAOSupport().loadList(query);
		}
		return null;  
	} 
	
	
	
	public RubricColumn searchByPositionAndId(Integer pos, Integer id) 
			throws KeyNotFoundException {
		if (pos != null && id != null) {
			//Change column name to whatever the name is in the bean
			SimpleSelectQuery query = new SimpleSelectQuery(getDAOSupport().getMap());
			//Change column name to whatever the name is in the bean
			query.addWhere("rubricId", id);
			query.addWhere("position", pos);
			List<RubricColumn> cols = getDAOSupport().loadList(query);
			if(cols.size() == 0 ){
				return null;
			}
			return cols.get(0);
			/*List<RubricColumn> allCols;
	        allCols = getDAOSupport().loadAll();
	        for (RubricColumn col : allCols){
	        	if(col.getPosition() == pos && col.getRubricId() == id){
	        		return col;
	        	}
	        }*/
		}
		return null;  
	}
	
	
    public RubricColumn loadById(long id) {
        RubricColumn rubric = null; 
        try {
        	 Id realId = Id.generateId(RubricColumn.DATA_TYPE, id);
        	 rubric = getDAOSupport().loadById(realId);
		} catch (PersistenceException e1) {
			LOGGER.debug("Failed to load courseID! ID: "+ id +" # " + e1);
			e1.printStackTrace();
		}
       
        LOGGER.debug("RubricColumn Loaded succsessfully "+ id +" # " + rubric);
        if(rubric!=null){
        	return rubric;
        }
        else return null;
    } 
    
  //Currently loads the last element in the list (last added)
    public RubricColumn loadLast() {
        List<RubricColumn> globalSettings;
        globalSettings = getDAOSupport().loadAll();
        if(globalSettings!=null&&!globalSettings.isEmpty()){
            //DEBUGGING
        	for (RubricColumn rubric : globalSettings) {
        		LOGGER.debug("A#"+ rubric);
			}
        return globalSettings.get(globalSettings.size()-1);
        }
        else return null;
    } 
    
    public void save(RubricColumn globalSettings) {
        System.out.println(globalSettings);        
                
        getDAOSupport().persist(globalSettings);
    }
    
    public boolean isInstalled(){
        List<RubricColumn> globalSettings = null;
        globalSettings = getDAOSupport().loadAll();
        
   
        if (globalSettings.size() > 0) {
            return true;
        }
        return false;
    }
    
    public void writeDefaults() {
    	RubricColumn globalSettings = new RubricColumn();
        
        getDAOSupport().persist(globalSettings);
        
    }
}
