/**
 * Copyright 2017 Anders Lunde, Robin Sjvoll
 *
 * This code is developed under an experimental paradigm, thus it is important to reuse it with caution. 
 */
package no.ntnu.es.bb.schema.rubric;

import java.util.List;

import org.apache.log4j.Logger;

import blackboard.persist.Id;
import blackboard.persist.PersistenceException;
import blackboard.persist.KeyNotFoundException;
import blackboard.persist.dao.impl.SimpleDAO;
import blackboard.persist.impl.SimpleSelectQuery;
import blackboard.util.StringUtil;

public class RubricDAO extends SimpleDAO<Rubric> {
	private static final Logger LOGGER = Logger.getLogger( RubricDAO.class.getName() );
	public RubricDAO() {
		super(Rubric.class);
	}

	public RubricDAO(Class<Rubric> cls) {
		super(cls);
	}

	public List<Rubric> loadAll() {
		return getDAOSupport().loadAll();
	}

	public List<Rubric> searchById(String id) 
			throws KeyNotFoundException {
		if (!StringUtil.isEmpty(id)) {
			SimpleSelectQuery query = new SimpleSelectQuery(getDAOSupport().getMap());
			//Change column name to whatever the name is in the bean
			query.addWhere("pk1", id);
			return getDAOSupport().loadList(query);
		}
		return null;  
	} 
	
	public Rubric searchByUserIDAndAssessmentID(Integer userID, Integer assessmentID) 
			throws KeyNotFoundException {
		LOGGER.debug("User ID and Assessment ID "+ userID +" # " + assessmentID);
		if (userID != 0 && assessmentID != 0) {
			SimpleSelectQuery query = new SimpleSelectQuery(getDAOSupport().getMap());
			//Change column name to whatever the name is in the bean
			query.addWhere("submissionUserId", userID);
			query.addWhere("peerAssessmentId", assessmentID);
//			return getDAOSupport().loadList(query).get(0);
			return getDAOSupport().load(query);
		}
		return null;  
	} 
	
	public List<Rubric> findAllWithAssessmentID(Integer assessmentID) 
			throws KeyNotFoundException {
		LOGGER.debug("Find all Assessments with ID: " + assessmentID);
		if (assessmentID != 0) {
			SimpleSelectQuery query = new SimpleSelectQuery(getDAOSupport().getMap());
			query.addWhere("peerAssessmentId", assessmentID);
			return getDAOSupport().loadList(query);
		}
		return null;  
	} 
	
	
    public Rubric loadById(long id) {
        Rubric rubric = null; 
        try {
        	 Id realId = Id.generateId(Rubric.DATA_TYPE, id);
        	 rubric = getDAOSupport().loadById(realId);
		} catch (PersistenceException e1) {
			LOGGER.debug("Failed to load courseID! ID: "+ id +" # " + e1);
			e1.printStackTrace();
		}
       
        LOGGER.debug("Rubric Loaded succsessfully "+ id +" # " + rubric);
        if(rubric!=null){
        	return rubric;
        }
        else return null;
    } 
    
  //Currently loads the last element in the list (last added)
    public Rubric loadLast() {
        List<Rubric> globalSettings;
        globalSettings = getDAOSupport().loadAll();
        if(globalSettings!=null&&!globalSettings.isEmpty()){
            //DEBUGGING
        	for (Rubric rubric : globalSettings) {
        		LOGGER.debug("A#"+ rubric);
			}
        return globalSettings.get(globalSettings.size()-1);
        }
        else return null;
    } 
    
    public void save(Rubric globalSettings) {
        System.out.println(globalSettings);        
                
        getDAOSupport().persist(globalSettings);
    }
    
    public boolean isInstalled(){
        List<Rubric> globalSettings = null;
        globalSettings = getDAOSupport().loadAll();
        
   
        if (globalSettings.size() > 0) {
            return true;
        }
        return false;
    }
    
    public void writeDefaults() {
    	Rubric globalSettings = new Rubric();
        
        getDAOSupport().persist(globalSettings);
        
    }
}
