/**
 * Copyright 2017 Anders Lunde, Robin Sjvoll
 *
 * This code is developed under an experimental paradigm, thus it is important to reuse it with caution. 
 */
package no.ntnu.es.bb.schema.rubric;

import java.util.List;

import org.apache.log4j.Logger;

import blackboard.persist.Id;
import blackboard.persist.PersistenceException;
import blackboard.persist.KeyNotFoundException;
import blackboard.persist.dao.impl.SimpleDAO;
import blackboard.persist.impl.SimpleSelectQuery;
import blackboard.util.StringUtil;

public class RubricRowDAO extends SimpleDAO<RubricRow> {
	private static final Logger LOGGER = Logger.getLogger( RubricRowDAO.class.getName() );
	public RubricRowDAO() {
		super(RubricRow.class);
	}

	public RubricRowDAO(Class<RubricRow> cls) {
		super(cls);
	}

	public List<RubricRow> loadAll() {
		return getDAOSupport().loadAll();
	}
	
	public List<RubricRow> searchByRubricId(Integer id) 
			throws KeyNotFoundException {
		if (id != null) {
			SimpleSelectQuery query = new SimpleSelectQuery(getDAOSupport().getMap());
			//Change column name to whatever the name is in the bean
			query.addWhere("rubricId", id);
			return getDAOSupport().loadList(query);
		}
		return null;  
	} 

	public List<RubricRow> searchById(String id) 
			throws KeyNotFoundException {
		if (!StringUtil.isEmpty(id)) {
			SimpleSelectQuery query = new SimpleSelectQuery(getDAOSupport().getMap());
			//Change column name to whatever the name is in the bean
			query.addWhere("pk1", id);
			return getDAOSupport().loadList(query);
		}
		return null;  
	} 
	
	public RubricRow searchByPositionAndId(Integer pos, Integer id) 
			throws KeyNotFoundException {
		if (pos != null && id != null) {
			//Change column name to whatever the name is in the bean
			SimpleSelectQuery query = new SimpleSelectQuery(getDAOSupport().getMap());
			//Change column name to whatever the name is in the bean
			query.addWhere("rubricId", id);
			query.addWhere("position", pos);
			List<RubricRow> rows = getDAOSupport().loadList(query);
			if(rows.size() == 0 ){
				return null;
			}
			return rows.get(0);
		}
		return null;  
	}
	
	
    public RubricRow loadById(long id) {
        RubricRow rubric = null; 
        try {
        	 Id realId = Id.generateId(RubricRow.DATA_TYPE, id);
        	 rubric = getDAOSupport().loadById(realId);
		} catch (PersistenceException e1) {
			LOGGER.debug("Failed to load courseID! ID: "+ id +" # " + e1);
			e1.printStackTrace();
		}
       
        LOGGER.debug("RubricRow Loaded succsessfully "+ id +" # " + rubric);
        if(rubric!=null){
        	return rubric;
        }
        else return null;
    } 
    
  //Currently loads the last element in the list (last added)
    public RubricRow loadLast() {
        List<RubricRow> globalSettings;
        globalSettings = getDAOSupport().loadAll();
        if(globalSettings!=null&&!globalSettings.isEmpty()){
            //DEBUGGING
        	for (RubricRow rubric : globalSettings) {
        		LOGGER.debug("A#"+ rubric);
			}
        return globalSettings.get(globalSettings.size()-1);
        }
        else return null;
    } 
    
    public void save(RubricRow globalSettings) {
        System.out.println(globalSettings);        
                
        getDAOSupport().persist(globalSettings);
    }
    
    public boolean isInstalled(){
        List<RubricRow> globalSettings = null;
        globalSettings = getDAOSupport().loadAll();
        
   
        if (globalSettings.size() > 0) {
            return true;
        }
        return false;
    }
    
    public void writeDefaults() {
    	RubricRow globalSettings = new RubricRow();
        
        getDAOSupport().persist(globalSettings);
        
    }
}
