/**
 * Copyright 2017 Anders Lunde, Robin Sjvoll
 *
 * This code is developed under an experimental paradigm, thus it is important to reuse it with caution. 
 */
package no.ntnu.es.bb.schema.rubric;

import java.util.List;

import org.apache.log4j.Logger;

import blackboard.persist.Id;
import blackboard.persist.PersistenceException;
import blackboard.persist.KeyNotFoundException;
import blackboard.persist.dao.impl.SimpleDAO;
import blackboard.persist.impl.SimpleSelectQuery;
import blackboard.util.StringUtil;

public class RubricRowEvalDAO extends SimpleDAO<RubricRowEval> {
	private static final Logger LOGGER = Logger.getLogger( RubricRowEvalDAO.class.getName() );
	public RubricRowEvalDAO() {
		super(RubricRowEval.class);
	}

	public RubricRowEvalDAO(Class<RubricRowEval> cls) {
		super(cls);
	}

	public List<RubricRowEval> loadAll() {
		return getDAOSupport().loadAll();
	}
	
	public List<RubricRowEval> searchByRubricId(Integer id) 
			throws KeyNotFoundException {
		if (id != null) {
			SimpleSelectQuery query = new SimpleSelectQuery(getDAOSupport().getMap());
			//Change column name to whatever the name is in the bean
			query.addWhere("rubricId", id);
			return getDAOSupport().loadList(query);
		}
		return null;  
	} 

	public List<RubricRowEval> searchById(String id) 
			throws KeyNotFoundException {
		if (!StringUtil.isEmpty(id)) {
			SimpleSelectQuery query = new SimpleSelectQuery(getDAOSupport().getMap());
			//Change column name to whatever the name is in the bean
			query.addWhere("pk1", id);
			return getDAOSupport().loadList(query);
		}
		return null;  
	} 
	
	public RubricRowEval searchByEvalIdAndRubricIdandPos(Integer evalId, Integer rubricId, Integer pos) 
			throws KeyNotFoundException {
		if (evalId != null && rubricId != null) {
			//Change column name to whatever the name is in the bean
			SimpleSelectQuery query = new SimpleSelectQuery(getDAOSupport().getMap());
			//Change column name to whatever the name is in the bean
			query.addWhere("evaluationId", evalId);
			query.addWhere("rubricId", rubricId);
			query.addWhere("rowPosition", pos);
			RubricRowEval rows = getDAOSupport().load(query);
			return rows;
		}
		return null;  
	}
	
	public List<RubricRowEval> searchByEvalIdAndRubricId(Integer evalId, Integer rubricId) 
			throws KeyNotFoundException {
		if (evalId != null && rubricId != null) {
			//Change column name to whatever the name is in the bean
			SimpleSelectQuery query = new SimpleSelectQuery(getDAOSupport().getMap());
			//Change column name to whatever the name is in the bean
			query.addWhere("evaluationId", evalId);
			query.addWhere("rubricId", rubricId);
			List<RubricRowEval> rows = getDAOSupport().loadList(query);
			return rows;
		}
		return null;  
	}
	
	
    public RubricRowEval loadById(long id) {
        RubricRowEval rubric = null; 
        try {
        	 Id realId = Id.generateId(RubricRowEval.DATA_TYPE, id);
        	 rubric = getDAOSupport().loadById(realId);
		} catch (PersistenceException e1) {
			LOGGER.debug("Failed to load courseID! ID: "+ id +" # " + e1);
			e1.printStackTrace();
		}
       
        LOGGER.debug("RubricRowEval Loaded succsessfully "+ id +" # " + rubric);
        if(rubric!=null){
        	return rubric;
        }
        else return null;
    } 
    
  //Currently loads the last element in the list (last added)
    public RubricRowEval loadLast() {
        List<RubricRowEval> globalSettings;
        globalSettings = getDAOSupport().loadAll();
        if(globalSettings!=null&&!globalSettings.isEmpty()){
            //DEBUGGING
        	for (RubricRowEval rubric : globalSettings) {
        		LOGGER.debug("A#"+ rubric);
			}
        return globalSettings.get(globalSettings.size()-1);
        }
        else return null;
    } 
    
    public void save(RubricRowEval globalSettings) {
        System.out.println(globalSettings);        
                
        getDAOSupport().persist(globalSettings);
    }
    
    public boolean isInstalled(){
        List<RubricRowEval> globalSettings = null;
        globalSettings = getDAOSupport().loadAll();
        
   
        if (globalSettings.size() > 0) {
            return true;
        }
        return false;
    }
    
    public void writeDefaults() {
    	RubricRowEval globalSettings = new RubricRowEval();
        
        getDAOSupport().persist(globalSettings);
        
    }
}
